@@ -273,25 +273,6 @@ def pad_examples(x, desired_batch_size):
   return np.concatenate([x, np.tile(x[-1], (batch_pad, 1))], axis=0)
 
 
-def per_host_sum_pmap(in_tree):
-  """Execute psum on in_tree"s leaves over one device per host."""
-  host2devices = collections.defaultdict(list)
-  for d in jax.devices():
-    host2devices[d.process_index].append(d)
-  devices = [host2devices[k][0] for k in host2devices]
-  host_psum = jax.pmap(lambda x: jax.lax.psum(x, "i"), "i", devices=devices)
-
-  def pre_pmap(xs):
-    return jax.tree_util.tree_map(
-        lambda x: jnp.broadcast_to(x, (1,) + x.shape), xs
-    )
-
-  def post_pmap(xs):
-    return jax.tree_util.tree_map(lambda x: x[0], xs)
-
-  return post_pmap(host_psum(pre_pmap(in_tree)))
-
-
 def tohost(x):
   """Collect batches from all devices to host and flatten batch dimensions."""
   n_device, n_batch, *remaining_dims = x.shape
@@ -515,11 +496,11 @@ def train_and_evaluate(config: ml_collections.ConfigDict, workdir: str):
           predict_step,
           in_axes=(
               0,
-              jax.tree_map(lambda x: None, state.params),
+              jax.tree_util.tree_map(lambda x: None, state.params),
               0,
               None,
               None,
-              jax.tree_map(lambda x: None, predict_config),
+              jax.tree_util.tree_map(lambda x: None, predict_config),
               None,
               None,
           ),
@@ -558,7 +539,7 @@ def train_and_evaluate(config: ml_collections.ConfigDict, workdir: str):
       # Shard data to devices and do a training step.
       with jax.profiler.StepTraceAnnotation("train", step_num=step):
         batch = next(train_iter)
-        batch = jax.tree_map(lambda x: jnp.array(x), batch)
+        batch = jax.tree_util.tree_map(lambda x: jnp.array(x), batch)
         state, metrics = jit_train_step(
             state, batch, train_config, learning_rate_fn, 0.0, dropout_rngs
         )
@@ -582,7 +563,7 @@ def train_and_evaluate(config: ml_collections.ConfigDict, workdir: str):
           )  # pylint: disable=cell-var-from-loop
           summary["learning_rate"] = lr
           summary["perplexity"] = jnp.clip(
-              jnp.exp(summary["loss"]), a_max=1.0e4
+              jnp.exp(summary["loss"]), max=1.0e4
           )
           summary = {"train_" + k: v for k, v in summary.items()}
           writer.write_scalars(step, summary)
@@ -598,7 +579,7 @@ def train_and_evaluate(config: ml_collections.ConfigDict, workdir: str):
           )
           # (clipped) perplexity after averaging log-perplexitie
           eval_results["perplexity"] = jnp.clip(
-              jnp.exp(eval_results["loss"]), a_max=1.0e4
+              jnp.exp(eval_results["loss"]), max=1.0e4
           )
           writer.write_scalars(
               step, {"eval_" + k: v for k, v in eval_results.items()}
