@@ -22,6 +22,12 @@ _T = TypeVar('_T')
 
 
 class Config:
+  flax_use_flaxlib: bool
+  flax_array_ref: bool
+  flax_pytree_module: bool
+  flax_max_repr_depth: int | None
+  flax_always_shard_variable: bool
+  flax_hijax_variable: bool
   # See https://google.github.io/pytype/faq.html.
   _HAS_DYNAMIC_ATTRIBUTES = True
 
@@ -62,6 +68,25 @@ class Config:
       raise LookupError(f'Unrecognized config option: {name}')
     self._values[name] = value
 
+  def __repr__(self):
+    values_repr = ', '.join(f'\n  {k}={v!r}' for k, v in self._values.items())
+    return f'Config({values_repr}\n)'
+
+  @contextmanager
+  def temp_flip_flag(self, var_name: str, var_value: bool):
+    """Context manager to temporarily flip feature flags for test functions.
+
+    Args:
+      var_name: the config variable name (without the 'flax_' prefix)
+      var_value: the boolean value to set var_name to temporarily
+    """
+    old_value = getattr(self, f'flax_{var_name}')
+    try:
+      self.update(f'flax_{var_name}', var_value)
+      yield
+    finally:
+      self.update(f'flax_{var_name}', old_value)
+
 
 config = Config()
 
@@ -117,6 +142,38 @@ def bool_flag(name: str, *, default: bool, help: str) -> FlagHolder[bool]:
   return fh
 
 
+def int_flag(name: str, *, default: int | None, help: str) -> FlagHolder[int]:
+  """Set up an integer flag.
+
+  Example::
+
+    num_foo = int_flag(
+        name='flax_num_foo',
+        default=42,
+        help='Number of foo.',
+    )
+
+  Now the ``FLAX_NUM_FOO`` shell environment variable can be used to
+  control the process-level value of the flag, in addition to using e.g.
+  ``config.update("flax_num_foo", 42)`` directly.
+
+  Args:
+    name: converted to lowercase to define the name of the flag. It is
+      converted to uppercase to define the corresponding shell environment
+      variable.
+    default: a default value for the flag.
+    help: used to populate the docstring of the returned flag holder object.
+
+  Returns:
+    A flag holder object for accessing the value of the flag.
+  """
+  name = name.lower()
+  config._add_option(name, static_int_env(name.upper(), default))
+  fh = FlagHolder[int](name, help)
+  setattr(Config, name, property(lambda _: fh.value, doc=help))
+  return fh
+
+
 def static_bool_env(varname: str, default: bool) -> bool:
   """Read an environment variable and interpret it as a boolean.
 
@@ -140,31 +197,33 @@ def static_bool_env(varname: str, default: bool) -> bool:
     return False
   else:
     raise ValueError(
-      'invalid truth value {!r} for environment {!r}'.format(val, varname)
+      f'invalid truth value {val!r} for environment {varname!r}'
     )
 
 
-@contextmanager
-def temp_flip_flag(var_name: str, var_value: bool):
-  """Context manager to temporarily flip feature flags for test functions.
+def static_int_env(varname: str, default: int | None) -> int | None:
+  """Read an environment variable and interpret it as an integer.
 
   Args:
-    var_name: the config variable name (without the 'flax_' prefix)
-    var_value: the boolean value to set var_name to temporarily
+    varname: the name of the variable
+    default: the default integer value
+  Returns:
+    integer return value derived from defaults and environment.
+  Raises: ValueError if the environment variable is not an integer.
   """
-  old_value = getattr(config, f'flax_{var_name}')
+  val = os.getenv(varname)
+  if val is None:
+    return default
   try:
-    config.update(f'flax_{var_name}', var_value)
-    yield
-  finally:
-    config.update(f'flax_{var_name}', old_value)
+    return int(val)
+  except ValueError:
+    raise ValueError(
+      f'invalid integer value {val!r} for environment {varname!r}'
+    ) from None
 
 
 # Flax Global Configuration Variables:
 
-# Whether to use the lazy rng implementation.
-flax_lazy_rng = static_bool_env('FLAX_LAZY_RNG', True)
-
 flax_filter_frames = bool_flag(
   name='flax_filter_frames',
   default=True,
@@ -204,3 +263,36 @@ flax_fix_rng = bool_flag(
     ' PRNG keys.'
   ),
 )
+
+flax_use_flaxlib = bool_flag(
+  name='flax_use_flaxlib',
+  default=False,
+  help='Whether to use flaxlib for C++ acceleration.',
+)
+flax_array_ref = bool_flag(
+  name='flax_array_ref',
+  default=False,
+  help='Whether to use array refs.',
+)
+flax_pytree_module = bool_flag(
+  name='flax_pytree_module',
+  default=True,
+  help='Whether Modules are pytrees by default or not.',
+)
+
+flax_max_repr_depth = int_flag(
+  name='flax_max_repr_depth',
+  default=None,
+  help='Maximum depth of reprs for nested flax objects. Default is None (no limit).',
+)
+
+flax_always_shard_variable = bool_flag(
+  name='flax_always_shard_variable',
+  default=True,
+  help='Whether a `nnx.Variable` should always automatically be sharded if it contains sharding annotations.',
+)
+flax_hijax_variable = bool_flag(
+  name='flax_hijax_variable',
+  default=False,
+  help='Whether to enable HiJAX support for `nnx.Variable`.',
+)
\ No newline at end of file
