@@ -1,4 +1,4 @@
-# Copyright 2022 The Flax Authors.
+# Copyright 2024 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,20 +22,22 @@ to keep track of how variables should be partitioned with ``jax.pjit``.
 """
 
 import abc
+import dataclasses
 import functools
-from typing import Any, Callable, Dict, Mapping, Tuple, TypeVar, Union
+from typing import Any, Generic, TypeVar
+from collections.abc import Callable
 
-from flax import errors
-from flax import struct
+from flax import errors, struct
+from flax.typing import LogicalNames
 import jax
-from jax.experimental import maps
-from jax.experimental import pjit
+from jax.interpreters import pxla
+
+A = TypeVar('A')
+B = TypeVar('B')
+TAxisMetadata = TypeVar('TAxisMetadata', bound='AxisMetadata[Any]')
 
 
-TAxisMetadata = Any # TypeVar('TAxisMetadata', bound='AxisMetadata')
-
-
-class AxisMetadata(metaclass=abc.ABCMeta):
+class AxisMetadata(Generic[A], metaclass=abc.ABCMeta):
   """Abstract base class for boxed Metadata.
 
   ``AxisMetadata`` enables arbitrary, per axis metadata for variables.
@@ -54,16 +56,16 @@ class AxisMetadata(metaclass=abc.ABCMeta):
   """
 
   @abc.abstractmethod
-  def unbox(self) -> Any:
+  def unbox(self) -> A:
     """Returns the content of the AxisMetadata box.
 
-    Note that unlike ``meta.unbox`` the unbox call should recursively unbox
+    Note that unlike ``meta.unbox`` the unbox call should not recursively unbox
     metadata. It should simply return value that it wraps directly even
     if that value itself is an instance of AxisMetadata.
 
     In practise, AxisMetadata subclasses should be registered as PyTree nodes to
     support passing instances to JAX and Flax APIs. The leaves returned for this
-    note should correspond to the value returned by unbox.
+    node should correspond to the value returned by unbox.
 
     Returns:
       The unboxed value.
@@ -71,11 +73,12 @@ class AxisMetadata(metaclass=abc.ABCMeta):
     pass
 
   @abc.abstractmethod
-  def replace_boxed(self, val: Any) -> TAxisMetadata:
+  def replace_boxed(self, val: B) -> 'AxisMetadata[B]':
     """Replaces the boxed value with the provided value.
 
     Args:
       val: The new value to be boxed by this AxisMetadata wrapper
+
     Returns:
       A new instance of the same type as self with `val` as the new ``unbox``
       content
@@ -83,8 +86,9 @@ class AxisMetadata(metaclass=abc.ABCMeta):
     pass
 
   @abc.abstractmethod
-  def add_axis(self: TAxisMetadata, index: int,
-               params: Dict[Any, Any]) -> TAxisMetadata:
+  def add_axis(
+      self: TAxisMetadata, index: int, params: dict[Any, Any]
+  ) -> TAxisMetadata:
     """Adds a new axis to the axis metadata.
 
     Note that add_axis and remove_axis should act as each other's inverse
@@ -96,6 +100,7 @@ class AxisMetadata(metaclass=abc.ABCMeta):
         that introduces the new axis (e.g.: ``nn.scan`` or ``nn.vmap``). The
         user passes this dictionary as the `metadata_param` argument to the
         transformation.
+
     Returns:
       A new instance of the same type as self and with the same ``unbox``
       content with updated axis metadata.
@@ -103,8 +108,9 @@ class AxisMetadata(metaclass=abc.ABCMeta):
     pass
 
   @abc.abstractmethod
-  def remove_axis(self: TAxisMetadata, index: int,
-                  params: Dict[Any, Any]) -> TAxisMetadata:
+  def remove_axis(
+      self: TAxisMetadata, index: int, params: dict[Any, Any]
+  ) -> TAxisMetadata:
     """Removes an axis from the axis metadata.
 
     Note that add_axis and remove_axis should act as each other's inverse
@@ -113,9 +119,10 @@ class AxisMetadata(metaclass=abc.ABCMeta):
     Args:
       index: The position of the axis that is to be removed
       params: An arbitrary dictionary of parameters passed by the transformation
-        that introduced the axis (e.g.: ``nn.scan`` or ``nn.vmap``). The
-        user passes this dictionary as the `metadata_param` argument to the
+        that introduced the axis (e.g.: ``nn.scan`` or ``nn.vmap``). The user
+        passes this dictionary as the `metadata_param` argument to the
         transformation.
+
     Returns:
       A new instance of the same type as self and with the same ``unbox``
       content with updated axis metadata.
@@ -128,22 +135,24 @@ def is_axis_metadata(val: Any) -> bool:
   return isinstance(val, AxisMetadata)
 
 
-def map_axis_meta(fn: Callable[[AxisMetadata], Any], tree: Any) -> Any:
+def map_axis_meta(fn: Callable[[AxisMetadata[Any]], Any], tree: Any) -> Any:
   """Maps over all PyTree nodes that are AxisMetadata instances."""
+
   def wrapper(x):
     if isinstance(x, AxisMetadata):
       return fn(x)
     else:
       return x
-  return jax.tree_map(wrapper, tree, is_leaf=is_axis_metadata)
+
+  return jax.tree_util.tree_map(wrapper, tree, is_leaf=is_axis_metadata)
 
 
-def add_axis(tree: Any, index: int, params: Dict[Any, Any]) -> Any:
+def add_axis(tree: Any, index: int, params: dict[Any, Any]) -> Any:
   """Add an axis to each AxisMetadata node in a PyTree."""
   return map_axis_meta(lambda x: x.add_axis(index, params), tree)
 
 
-def remove_axis(tree: Any, index: int, params: Dict[Any, Any]) -> Any:
+def remove_axis(tree: Any, index: int, params: dict[Any, Any]) -> Any:
   """Remove an axis from each AxisMetadata node in a PyTree."""
   return map_axis_meta(lambda x: x.remove_axis(index, params), tree)
 
@@ -155,25 +164,37 @@ def unbox(tree: Any) -> Any:
 
 def replace_boxed(tree: Any, updates: Any) -> Any:
   """Updates all AxisMetadata boxes with the values in updates."""
+
   def inner_update(c, v):
     if isinstance(c, AxisMetadata):
       return c.replace_boxed(replace_boxed(c.unbox(), v))
     else:
       return v
-  return jax.tree_map(inner_update, tree, updates, is_leaf=is_axis_metadata)
+
+  return jax.tree_util.tree_map(
+      inner_update, tree, updates, is_leaf=is_axis_metadata
+  )
 
 
 PARTITION_NAME = 'partition_name'
-LogicalNames = Tuple[Union[str, None], ...]
 
 
-def _global_mesh_defined() -> bool:
-  """Checks if global xmap/pjit mesh resource environment is defined."""
-  maps_env = maps.thread_resources.env
-  return maps_env.physical_mesh.devices.shape != ()  # pylint: disable=g-explicit-bool-comparison
+def get_global_mesh() -> jax.sharding.AbstractMesh | jax.sharding.Mesh | None:
+  mesh = jax.sharding.get_abstract_mesh()
+  if mesh.empty:
+    mesh = pxla.thread_resources.env.physical_mesh
+    if mesh.empty:
+      return None
+  return mesh
 
 
-class Partitioned(struct.PyTreeNode, AxisMetadata):
+def global_mesh_defined() -> bool:
+  """Checks if global mesh resource environment is defined."""
+  mesh = get_global_mesh()
+  return mesh is not None
+
+
+class Partitioned(struct.PyTreeNode, AxisMetadata[A]):
   """Wrapper for partitioning metadata.
 
   ``Partitioned`` is used to extend variables with partitioning information
@@ -199,13 +220,13 @@ class Partitioned(struct.PyTreeNode, AxisMetadata):
     mlp = MLP(4096)
     x = jnp.ones((8 * 1024, 1024))
     # use eval_shape to get the Partitioned instances for the variables.
-    # this way we can determinte the PartitionSpecs for the init variables
+    # this way we can determine the PartitionSpecs for the init variables
     # before we call the init fn.
     var_spec = nn.get_partition_spec(
-        jax.eval_shape(mlp.init, random.PRNGKey(0), x))
+        jax.eval_shape(mlp.init, random.key(0), x))
     init_fn = mesh(pjit(mlp.init,
                         (None, PartitionSpec("data", "model")), var_spec))
-    variables = init_fn(random.PRNGKey(0), x)
+    variables = init_fn(random.key(0), x)
     apply_fn = mesh(pjit(
         mlp.apply,
         (var_spec, PartitionSpec("data", "model")),
@@ -227,36 +248,42 @@ class Partitioned(struct.PyTreeNode, AxisMetadata):
           body, variable_axes={"params": 0}, split_rngs={"params": 0}, length=8,
           metadata_params={nn.meta.PARTITION_NAME: "layers"})(self, x)
       return c
-
   """
+
   value: Any
   names: LogicalNames = struct.field(pytree_node=False)
+  mesh: jax.sharding.Mesh | None = struct.field(
+      default=None, pytree_node=False
+  )
 
-  def unbox(self, apply_constraint=True) -> Any:
+  def unbox(self, apply_constraint=True) -> A:
     """Returns the wrapped value with the partitioning applied as a sharding constraint."""
-    if apply_constraint and _global_mesh_defined():
-      return pjit.with_sharding_constraint(
-          self.value, self.get_partition_spec())
+    if apply_constraint and (global_mesh_defined() or self.mesh is not None):
+      axis_resource = self.get_partition_spec()
+      if self.mesh is not None:
+        sharding = jax.sharding.NamedSharding(self.mesh, axis_resource)
+        return jax.lax.with_sharding_constraint(self.value, sharding)
+      return jax.lax.with_sharding_constraint(self.value, axis_resource)
     else:
       return self.value
 
-  def replace_boxed(self, val: Any) -> TAxisMetadata:
-    return self.replace(value=val)
+  def replace_boxed(self, val: B) -> 'Partitioned[B]':
+    return self.replace(value=val)  # type: ignore
 
-  def _get_partition_name(self, params: Dict[Any, Any]) -> str:
+  def _get_partition_name(self, params: dict[Any, Any]) -> str:
     if PARTITION_NAME not in params:
       raise errors.PartitioningUnspecifiedError(self)
     return params[PARTITION_NAME]
 
-  def add_axis(self, index: int, params: Dict[Any, Any]) -> TAxisMetadata:
+  def add_axis(self, index: int, params: dict[Any, Any]) -> 'Partitioned[A]':
     axis_name = self._get_partition_name(params)
     names = list(self.names)
     while len(names) < index:
-      names.append(None) # type: ignore
-    names.insert(index, axis_name) # type: ignore
+      names.append(None)  # type: ignore
+    names.insert(index, axis_name)  # type: ignore
     return self.replace(names=tuple(names))
 
-  def remove_axis(self, index: int, params: Dict[Any, Any]) -> TAxisMetadata:
+  def remove_axis(self, index: int, params: dict[Any, Any]) -> 'Partitioned[A]':
     axis_name = self._get_partition_name(params)
     names = list(self.names)
     assert names.pop(index) == axis_name
@@ -266,36 +293,82 @@ class Partitioned(struct.PyTreeNode, AxisMetadata):
     """Returns the ``Partitionspec`` for this partitioned value."""
     return jax.sharding.PartitionSpec(*self.names)
 
+  def get_sharding(self, mesh: jax.sharding.Mesh) -> jax.sharding.Sharding:
+    """Returns the ``NamedSharding`` for this partitioned value."""
+    return jax.sharding.NamedSharding(mesh, self.get_partition_spec())
+
+  def to_nnx_metadata(self) -> dict[str, Any]:
+    """Return a dict of metadata that can translate into an `nnx.Variable`."""
+    metadata = dict(vars(self))
+    metadata['sharding_names'] = metadata.pop('names')
+    return metadata
+
+  @classmethod
+  def from_nnx_metadata(cls, metadata: dict[str, Any]):
+    """Given a dict of `nnx.Variable` format metadata, create a `nn.Partitioned`."""
+    metadata['names'] = metadata.pop('sharding_names')
+    fields = {x.name for x in dataclasses.fields(cls)}
+    return cls(**{k: v for k, v in metadata.items() if k in fields})
+
 
 def with_partitioning(
     fn: Callable[..., Any],
-    names: LogicalNames) ->  Callable[..., Partitioned]:
+    names: LogicalNames,
+    mesh: jax.sharding.Mesh | None = None,
+) -> Callable[..., Partitioned[Any]]:
   """Wraps a function's return value with Partitioned.
 
   Example::
 
-    kernel_init = with_partitioning(
-        nn.initializers.lecun_normal, (None, "data"))
-    partitioned_dense = nn.Dense(features, kernel_init=kernel_init)
+    >>> import flax.linen as nn
+    >>> kernel_init = nn.with_partitioning(
+    ...     nn.initializers.lecun_normal(), (None, "data"))
+    >>> partitioned_dense = nn.Dense(features=3, kernel_init=kernel_init)
 
   Args:
     fn: The function to be wrapped. Typically this is an initializer.
     names: The logical axis passed to ``Partitioned``.
+    mesh: The mesh to use for the partitioning. If None, the global mesh
+      resource is used if available.
+
   Returns:
     A function wrapping ``fn`` that will return an instance of ``Partitioned``.
   """
+
   @functools.wraps(fn)
   def wrapper(*args, **kwargs):
-    return Partitioned(fn(*args, **kwargs), names)
+    return Partitioned(fn(*args, **kwargs), names, mesh=mesh)
+
   return wrapper
 
 
+def _get_leaf_pspec(x: Any) -> jax.sharding.PartitionSpec | None:
+  if hasattr(x, 'get_partition_spec'):
+    return x.get_partition_spec()
+  # Unboxed arrays, which should be replicated across all devices
+  elif hasattr(x, 'shape'):
+    return jax.sharding.PartitionSpec()
+  else:
+    return None
+
+
 def get_partition_spec(tree: Any) -> Any:
   """Extracts a PartitionSpec tree from a PyTree containing ``Partitioned`` values."""
-  def f(x):
-    if isinstance(x, Partitioned):
-      return x.get_partition_spec()
-    else:
+  return jax.tree_util.tree_map(
+      _get_leaf_pspec, tree, is_leaf=lambda x: isinstance(x, AxisMetadata)
+  )
+
+
+def get_sharding(tree: Any, mesh: jax.sharding.Mesh) -> Any:
+  """Extracts a jax.sharding tree from a PyTree containing ``Partitioned`` values and a mesh."""
+  def f(x: Any) -> jax.sharding.Sharding | None:
+    if hasattr(x, 'get_sharding'):
+      return x.get_sharding(mesh)
+    pspec = _get_leaf_pspec(x)
+    if pspec is None:
       return None
-  return jax.tree_map(f, tree,
-                      is_leaf=lambda x: isinstance(x, Partitioned))
+    return jax.sharding.NamedSharding(mesh, pspec)
+
+  return jax.tree_util.tree_map(
+      f, tree, is_leaf=lambda x: isinstance(x, AxisMetadata)
+  )
