@@ -56,8 +56,21 @@ class FlaxError(Exception):
     )
     module_name = self.__class__.__module__
     class_name = self.__class__.__name__
-    error_msg = f'{message} ({error_page}#{module_name}.{class_name})'
-    super().__init__(error_msg)
+    if error_page not in message: # do not add a FlaxError link on unpickling
+      message = f'{message} ({error_page}#{module_name}.{class_name})'
+    super().__init__(message)
+
+  def __reduce__(self):
+   return (FlaxError, (str(self),))
+
+
+#################################################
+# NNX errors                                    #
+#################################################
+
+
+class TraceContextError(FlaxError):
+  pass
 
 
 #################################################
@@ -270,9 +283,9 @@ class ScopeParamShapeError(FlaxError):
 
   def __init__(self, param_name, scope_path, value_shape, init_shape):
     super().__init__(
-      f'Initializer expected to generate shape {init_shape} '
-      f'but got shape {value_shape} instead for parameter '
-      f'"{param_name}" in "{scope_path}".'
+        f'For parameter "{param_name}" in "{scope_path}", the given '
+        f'initializer is expected to generate shape {init_shape}, but the '
+        f'existing parameter it received has shape {value_shape}.'
     )
 
 
@@ -338,6 +351,22 @@ class ModifyScopeVariableError(FlaxError):
     )
 
 
+class ImmutableVariableError(FlaxError):
+  """You cannot update a variable that is marked as immutable.
+
+  This error occurs when attempting to modify a Variable that has been set to
+  'immutable' mode. Variables in immutable mode are read-only and cannot be
+  changed after creation.
+
+  To fix this error, either:
+  1. Use a different variable mode (e.g., 'qdd' or 'pytree')
+  2. Or ensure you're not trying to modify the variable's value
+  """
+
+  def __init__(self, message):
+    super().__init__(message)
+
+
 class JaxTransformError(FlaxError):
   """JAX transforms and Flax modules cannot be mixed.
 
@@ -687,6 +716,24 @@ class CallUnbindOnUnboundModuleError(FlaxError):
   def __init__(self):
     super().__init__("Can't call `unbind()` on unbound modules")
 
+class CallShareScopeOnUnboundModuleError(FlaxError):
+  """This error occurs when you are trying to call ``nn.share_scope`` on an unbound
+  Module. For instance, when you try to use ``nn.share_scope`` at the top-level::
+
+    from flax import linen as nn
+
+    class CustomDense(nn.Dense):
+      def __call__(self, x):
+        return super().__call__(x) + 1
+
+    custom_dense = CustomDense(5)
+    dense = nn.Dense(5)  # has the parameters
+
+    nn.share_scope(custom_dense, dense) # <-- ERROR!
+  """
+
+  def __init__(self):
+    super().__init__("Can't call `share_scope` on unbound modules")
 
 class InvalidInstanceModuleError(FlaxError):
   """This error occurs when you are trying to call ``.init()``, ``.init_with_output()``, ``.apply()`` or ``.bind()``
