@@ -12,32 +12,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Copyright 2024 The Flax Authors.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 """The Flax Module system."""
 
 
 # pylint: disable=g-multiple-import,useless-import-alias
 # re-export commonly used modules and functions
-from ..core import (
+from flax.core import (
     DenyList as DenyList,
     FrozenDict as FrozenDict,
     broadcast as broadcast,
     meta as meta,
 )
-from ..core.meta import (
+from flax.core.meta import (
     PARTITION_NAME as PARTITION_NAME,
     Partitioned as Partitioned,
     get_partition_spec as get_partition_spec,
@@ -45,8 +31,12 @@ from ..core.meta import (
     unbox as unbox,
     with_partitioning as with_partitioning,
 )
+from flax.core.spmd import (
+    get_logical_axis_rules as get_logical_axis_rules,
+    logical_axis_rules as logical_axis_rules,
+    set_logical_axis_rules as set_logical_axis_rules,
+)
 from .activation import (
-    GeGLU as GeGLU,
     PReLU as PReLU,
     celu as celu,
     elu as elu,
@@ -86,7 +76,13 @@ from .attention import (
 )
 from .batch_apply import BatchApply as BatchApply
 from .combinators import Sequential as Sequential
-from .fp8_ops import Fp8DotGeneralOp as Fp8DotGeneralOp
+from .fp8_ops import (
+    Fp8DirectDotGeneralOp as Fp8DirectDotGeneralOp,
+    Fp8DotGeneral as Fp8DotGeneral,
+    Fp8DotGeneralOp as Fp8DotGeneralOp,
+    Fp8Einsum as Fp8Einsum,
+    NANOOFp8DotGeneralOp as NANOOFp8DotGeneralOp,
+)
 from .initializers import (
     ones_init as ones_init,
     ones as ones,
@@ -99,14 +95,15 @@ from .linear import (
     Conv as Conv,
     DenseGeneral as DenseGeneral,
     Dense as Dense,
+    Einsum as Einsum,
     Embed as Embed,
 )
 from .module import (
     Module as Module,
     Variable as Variable,
     apply as apply,
-    compact as compact,
     compact_name_scope as compact_name_scope,
+    compact as compact,
     disable_named_call as disable_named_call,
     enable_named_call as enable_named_call,
     init_with_output as init_with_output,
@@ -115,6 +112,7 @@ from .module import (
     merge_param as merge_param,
     nowrap as nowrap,
     override_named_call as override_named_call,
+    share_scope as share_scope,
 )
 from .normalization import (
     BatchNorm as BatchNorm,
@@ -123,27 +121,25 @@ from .normalization import (
     LayerNorm as LayerNorm,
     RMSNorm as RMSNorm,
     SpectralNorm as SpectralNorm,
-    WeightNorm as WeightNorm
+    WeightNorm as WeightNorm,
 )
 from .pooling import (avg_pool as avg_pool, max_pool as max_pool, pool as pool)
 from .recurrent import (
     Bidirectional as Bidirectional,
     ConvLSTMCell as ConvLSTMCell,
     GRUCell as GRUCell,
-    MGUCell as MGUCell,
     LSTMCell as LSTMCell,
+    MGUCell as MGUCell,
     OptimizedLSTMCell as OptimizedLSTMCell,
     RNNCellBase as RNNCellBase,
     RNN as RNN,
+    SimpleCell as SimpleCell,
 )
 from .spmd import (
     LogicallyPartitioned as LogicallyPartitioned,
-    get_logical_axis_rules as get_logical_axis_rules,
-    logical_axis_rules as logical_axis_rules,
     logical_to_mesh,
     logical_to_mesh_axes,
     logical_to_mesh_sharding,
-    set_logical_axis_rules as set_logical_axis_rules,
     with_logical_constraint,
     with_logical_partitioning as with_logical_partitioning,
 )
@@ -154,6 +150,8 @@ from .transforms import (
     checkpoint as checkpoint,
     cond as cond,
     custom_vjp as custom_vjp,
+    fold_rngs as fold_rngs,
+    grad as grad,
     jit as jit,
     jvp as jvp,
     map_variables as map_variables,
@@ -162,9 +160,8 @@ from .transforms import (
     remat as remat,
     scan as scan,
     switch as switch,
-    vjp as vjp,
-    grad as grad,
     value_and_grad as value_and_grad,
+    vjp as vjp,
     vmap as vmap,
     while_loop as while_loop,
 )
