@@ -16,7 +16,8 @@
 
 import dataclasses
 import functools
-from typing import Any, Iterable, Optional, Tuple
+from typing import Any
+from collections.abc import Iterable
 
 import jax
 import jax.numpy as jnp
@@ -41,11 +42,11 @@ merge_param = module.merge_param
 map_variables = transforms.map_variables
 
 
-def _canonicalize_axes(rank: int, axes: Axes) -> Tuple[int, ...]:
+def _canonicalize_axes(rank: int, axes: Axes) -> tuple[int, ...]:
   """Returns a tuple of deduplicated, sorted, and positive axes."""
   if not isinstance(axes, Iterable):
     axes = (axes,)
-  return tuple(set([rank + axis if axis < 0 else axis for axis in axes]))
+  return tuple({rank + axis if axis < 0 else axis for axis in axes})
 
 
 def _abs_sq(x):
@@ -57,19 +58,21 @@ def _abs_sq(x):
 
 
 def _compute_stats(
-  x: Array,
-  axes: Axes,
-  dtype: Optional[Dtype],
-  axis_name: Optional[str] = None,
-  axis_index_groups: Any = None,
-  use_mean: bool = True,
-  use_fast_variance: bool = True,
-  mask: Optional[Array] = None,
+    x: Array,
+    axes: Axes,
+    dtype: Dtype | None,
+    axis_name: str | None = None,
+    axis_index_groups: Any = None,
+    use_mean: bool = True,
+    use_fast_variance: bool = True,
+    mask: Array | None = None,
+    force_float32_reductions=True,
 ):
   """Computes mean and variance statistics.
 
   This implementation takes care of a few important details:
-  - Computes in float32 precision for stability in half precision training.
+  - By default, computes in float32 precision for stability
+    in half precision training.
   - If `use_fast_variance` is `True`, mean and variance are computed using
     Var = E[|x|^2] - |E[x]|^2, instead of Var = E[|x - E[x]|^2]), in a single
     XLA fusion.
@@ -87,14 +90,16 @@ def _compute_stats(
       this is only used for pmap and shard map. For SPMD jit, you do not need to
       manually synchronize. Just make sure that the axes are correctly annotated
       and XLA:SPMD will insert the necessary collectives.
-    axis_index_groups: Optional axis indices.
+    axis_index_groups: Optional groups of indices within that named axis.
     use_mean: If true, calculate the mean from the input and use it when
       computing the variance. If false, set the mean to zero and compute the
       variance without subtracting the mean.
     use_fast_variance: If true, use a faster, but less numerically stable,
       calculation for the variance.
-    mask: Binary array of shape broadcastable to `inputs` tensor, indicating
-      the positions for which the mean and variance should be computed.
+    mask: Binary array of shape broadcastable to `inputs` tensor, indicating the
+      positions for which the mean and variance should be computed.
+    force_float32_reductions: If false, this will skip float32 promotion and use
+      the input dtype or inherited dtype from ``x``.
 
   Returns:
     A pair ``(mean, var)``.
@@ -103,11 +108,17 @@ def _compute_stats(
     dtype = jnp.result_type(x)
   # promote x to at least float32, this avoids half precision computation
   # but preserves double or complex floating points
-  dtype = jnp.promote_types(dtype, jnp.float32)
-  x = jnp.asarray(x, dtype)
+  if force_float32_reductions:
+    dtype = jnp.promote_types(dtype, jnp.float32)
+  if isinstance(x, jax.Array):
+    x = x.astype(dtype)
+  else:
+    x = jnp.asarray(x, dtype)
   axes = _canonicalize_axes(x.ndim, axes)
 
   def maybe_distributed_mean(*xs, mask=None):
+    if mask is not None:
+      mask = jnp.asarray(mask, dtype=bool)
     mus = tuple(x.mean(axes, where=mask) for x in xs)
     if axis_name is None:
       return mus if len(xs) > 1 else mus[0]
@@ -147,13 +158,14 @@ def _normalize(
   var: Array,
   reduction_axes: Axes,
   feature_axes: Axes,
-  dtype: Optional[Dtype],
+  dtype: Dtype | None,
   param_dtype: Dtype,
   epsilon: float,
   use_bias: bool,
   use_scale: bool,
   bias_init: Initializer,
   scale_init: Initializer,
+  force_float32_reductions: bool = True
 ):
   """Normalizes the input of a normalization layer and optionally applies a learned scale and bias.
 
@@ -173,6 +185,9 @@ def _normalize(
     use_scale: If true, scale the output.
     bias_init: Initialization function for the bias term.
     scale_init: Initialization function for the scaling function.
+    force_float32_reductions: If false, the scale and bias parameters use the
+      param_dtype. Otherwise, they will have at least float32 precision due to
+      the mean and var being promoted to float32.
 
   Returns:
     The normalized input.
@@ -194,6 +209,8 @@ def _normalize(
     scale = mdl.param(
       'scale', scale_init, reduced_feature_shape, param_dtype
     ).reshape(feature_shape)
+    if not force_float32_reductions:
+      scale = jnp.asarray(scale, param_dtype)
     mul *= scale
     args.append(scale)
   y *= mul
@@ -201,6 +218,8 @@ def _normalize(
     bias = mdl.param(
       'bias', bias_init, reduced_feature_shape, param_dtype
     ).reshape(feature_shape)
+    if not force_float32_reductions:
+      bias = jnp.asarray(bias, param_dtype)
     y += bias
     args.append(bias)
   dtype = dtypes.canonicalize_dtype(*args, dtype=dtype)
@@ -241,11 +260,12 @@ class BatchNorm(Module):
 
     >>> x = jax.random.normal(jax.random.key(0), (5, 6))
     >>> variables = BN.init(jax.random.key(1), x, use_running_average=False)
-    >>> jax.tree_map(jnp.shape, variables)
+    >>> jax.tree_util.tree_map(jnp.shape, variables)
     {'batch_stats': {'mean': (6,), 'var': (6,)}, 'params': {'bias': (6,), 'scale': (6,)}}
 
   We then update the batch_stats during training by specifying that the
-  ``batch_stats`` collection is mutable in the ``apply`` method for our module.::
+  ``batch_stats`` collection is mutable in the ``apply`` method for our
+  module.::
 
     >>> y, new_batch_stats = BN.apply(variables, x, mutable=['batch_stats'], use_running_average=False)
 
@@ -280,35 +300,42 @@ class BatchNorm(Module):
       representing subsets of devices to reduce over (default: None). For
       example, ``[[0, 1], [2, 3]]`` would independently batch-normalize over the
       examples on the first two and last two devices. See ``jax.lax.psum`` for
-      more details.
+      more details. This argument is currently not supported for SPMD jit.
     use_fast_variance: If true, use a faster, but less numerically stable,
       calculation for the variance.
   """
 
-  use_running_average: Optional[bool] = None
+  use_running_average: bool | None = None
   axis: int = -1
   momentum: float = 0.99
   epsilon: float = 1e-5
-  dtype: Optional[Dtype] = None
+  dtype: Dtype | None = None
   param_dtype: Dtype = jnp.float32
   use_bias: bool = True
   use_scale: bool = True
   bias_init: Initializer = initializers.zeros
   scale_init: Initializer = initializers.ones
-  axis_name: Optional[str] = None
+  axis_name: str | None = None
   axis_index_groups: Any = None
   use_fast_variance: bool = True
+  force_float32_reductions: bool = True
 
   @compact
-  def __call__(self, x, use_running_average: Optional[bool] = None, mask=None):
+  def __call__(
+      self,
+      x,
+      use_running_average: bool | None = None,
+      *,
+      mask: jax.Array | None = None,
+  ):
     """Normalizes the input using batch statistics.
 
-    NOTE:
-    During initialization (when ``self.is_initializing()`` is ``True``) the running
-    average of the batch statistics will not be updated. Therefore, the inputs
-    fed during initialization don't need to match that of the actual input
-    distribution and the reduction axis (set with ``axis_name``) does not have
-    to exist.
+    .. note::
+      During initialization (when ``self.is_initializing()`` is ``True``) the running
+      average of the batch statistics will not be updated. Therefore, the inputs
+      fed during initialization don't need to match that of the actual input
+      distribution and the reduction axis (set with ``axis_name``) does not have
+      to exist.
 
     Args:
       x: the input to be normalized.
@@ -329,26 +356,45 @@ class BatchNorm(Module):
     feature_shape = [x.shape[ax] for ax in feature_axes]
 
     ra_mean = self.variable(
-      'batch_stats',
-      'mean',
-      lambda s: jnp.zeros(s, jnp.float32),
-      feature_shape,
+        'batch_stats',
+        'mean',
+        lambda s: jnp.zeros(
+            s,
+            jnp.float32 if self.force_float32_reductions else self.param_dtype,
+        ),
+        feature_shape,
     )
     ra_var = self.variable(
-      'batch_stats', 'var', lambda s: jnp.ones(s, jnp.float32), feature_shape
+        'batch_stats',
+        'var',
+        lambda s: jnp.ones(
+            s,
+            jnp.float32 if self.force_float32_reductions else self.param_dtype,
+        ),
+        feature_shape,
     )
 
     if use_running_average:
-      mean, var = ra_mean.value, ra_var.value
+      mean = (
+          ra_mean.value
+          if self.force_float32_reductions
+          else jnp.asarray(ra_mean.value, self.param_dtype)
+      )
+      var = (
+          ra_var.value
+          if self.force_float32_reductions
+          else jnp.asarray(ra_var.value, self.param_dtype)
+      )
     else:
       mean, var = _compute_stats(
-        x,
-        reduction_axes,
-        dtype=self.dtype,
-        axis_name=self.axis_name if not self.is_initializing() else None,
-        axis_index_groups=self.axis_index_groups,
-        use_fast_variance=self.use_fast_variance,
-        mask=mask,
+          x,
+          reduction_axes,
+          dtype=self.dtype,
+          axis_name=self.axis_name if not self.is_initializing() else None,
+          axis_index_groups=self.axis_index_groups,
+          use_fast_variance=self.use_fast_variance,
+          mask=mask,
+          force_float32_reductions=self.force_float32_reductions,
       )
 
       if not self.is_initializing():
@@ -371,6 +417,7 @@ class BatchNorm(Module):
       self.use_scale,
       self.bias_init,
       self.scale_init,
+      self.force_float32_reductions,
     )
 
 
@@ -382,9 +429,10 @@ class LayerNorm(Module):
   i.e. applies a transformation that maintains the mean activation within
   each example close to 0 and the activation standard deviation close to 1.
 
-  NOTE: This normalization operation is identical to InstanceNorm and GroupNorm;
-  the difference is simply which axes are reduced and the shape of the feature axes
-  (i.e. the shape of the learnable scale and bias parameters).
+  .. note::
+    This normalization operation is identical to InstanceNorm and GroupNorm;
+    the difference is simply which axes are reduced and the shape of the feature
+    axes (i.e. the shape of the learnable scale and bias parameters).
 
   Example usage::
 
@@ -430,13 +478,13 @@ class LayerNorm(Module):
       representing subsets of devices to reduce over (default: None). For
       example, ``[[0, 1], [2, 3]]`` would independently batch-normalize over the
       examples on the first two and last two devices. See ``jax.lax.psum`` for
-      more details.
+      more details. This argument is currently not supported for SPMD jit.
     use_fast_variance: If true, use a faster, but less numerically stable,
       calculation for the variance.
   """
 
   epsilon: float = 1e-6
-  dtype: Optional[Dtype] = None
+  dtype: Dtype | None = None
   param_dtype: Dtype = jnp.float32
   use_bias: bool = True
   use_scale: bool = True
@@ -444,12 +492,13 @@ class LayerNorm(Module):
   scale_init: Initializer = initializers.ones
   reduction_axes: Axes = -1
   feature_axes: Axes = -1
-  axis_name: Optional[str] = None
+  axis_name: str | None = None
   axis_index_groups: Any = None
   use_fast_variance: bool = True
+  force_float32_reductions: bool = True
 
   @compact
-  def __call__(self, x, mask=None):
+  def __call__(self, x, *, mask: jax.Array | None = None):
     """Applies layer normalization on the input.
 
     Args:
@@ -461,13 +510,14 @@ class LayerNorm(Module):
       Normalized inputs (the same shape as inputs).
     """
     mean, var = _compute_stats(
-      x,
-      self.reduction_axes,
-      self.dtype,
-      self.axis_name,
-      self.axis_index_groups,
-      use_fast_variance=self.use_fast_variance,
-      mask=mask,
+        x,
+        self.reduction_axes,
+        self.dtype,
+        self.axis_name,
+        self.axis_index_groups,
+        use_fast_variance=self.use_fast_variance,
+        mask=mask,
+        force_float32_reductions=self.force_float32_reductions,
     )
 
     return _normalize(
@@ -484,6 +534,7 @@ class LayerNorm(Module):
       self.use_scale,
       self.bias_init,
       self.scale_init,
+      self.force_float32_reductions,
     )
 
 
@@ -529,21 +580,25 @@ class RMSNorm(Module):
       representing subsets of devices to reduce over (default: None). For
       example, ``[[0, 1], [2, 3]]`` would independently batch-normalize over the
       examples on the first two and last two devices. See ``jax.lax.psum`` for
-      more details.
+      more details. This argument is currently not supported for SPMD jit.
+    use_fast_variance: If true, use a faster, but less numerically stable,
+      calculation for the variance.
   """
 
   epsilon: float = 1e-6
-  dtype: Optional[Dtype] = None
+  dtype: Dtype | None = None
   param_dtype: Dtype = jnp.float32
   use_scale: bool = True
   scale_init: Initializer = initializers.ones
   reduction_axes: Axes = -1
   feature_axes: Axes = -1
-  axis_name: Optional[str] = None
+  axis_name: str | None = None
   axis_index_groups: Any = None
+  use_fast_variance: bool = True
+  force_float32_reductions: bool = True
 
   @compact
-  def __call__(self, x, mask=None):
+  def __call__(self, x, *, mask: jax.Array | None = None):
     """Applies RMS layer normalization on the input.
 
     Args:
@@ -555,13 +610,15 @@ class RMSNorm(Module):
       Normalized inputs (the same shape as inputs).
     """
     mean, var = _compute_stats(
-      x,
-      self.reduction_axes,
-      self.dtype,
-      self.axis_name,
-      self.axis_index_groups,
-      use_mean=False,
-      mask=mask,
+        x,
+        self.reduction_axes,
+        self.dtype,
+        self.axis_name,
+        self.axis_index_groups,
+        use_mean=False,
+        use_fast_variance=self.use_fast_variance,
+        mask=mask,
+        force_float32_reductions=self.force_float32_reductions,
     )
 
     return _normalize(
@@ -578,6 +635,7 @@ class RMSNorm(Module):
       self.use_scale,
       initializers.zeros,
       self.scale_init,
+      self.force_float32_reductions,
     )
 
 
@@ -591,8 +649,9 @@ class GroupNorm(Module):
   The user should either specify the total number of channel groups or the
   number of channels per group.
 
-  NOTE: LayerNorm is a special case of GroupNorm where ``num_groups=1``, and
-  InstanceNorm is a special case of GroupNorm where ``group_size=1``.
+  .. note::
+    LayerNorm is a special case of GroupNorm where ``num_groups=1``, and
+    InstanceNorm is a special case of GroupNorm where ``group_size=1``.
 
   Example usage::
 
@@ -644,27 +703,28 @@ class GroupNorm(Module):
       representing subsets of devices to reduce over (default: None). For
       example, ``[[0, 1], [2, 3]]`` would independently batch-normalize over the
       examples on the first two and last two devices. See ``jax.lax.psum`` for
-      more details.
+      more details. This argument is currently not supported for SPMD jit.
     use_fast_variance: If true, use a faster, but less numerically stable,
       calculation for the variance.
   """
 
-  num_groups: Optional[int] = 32
-  group_size: Optional[int] = None
+  num_groups: int | None = 32
+  group_size: int | None = None
   epsilon: float = 1e-6
-  dtype: Optional[Dtype] = None
+  dtype: Dtype | None = None
   param_dtype: Dtype = jnp.float32
   use_bias: bool = True
   use_scale: bool = True
   bias_init: Initializer = initializers.zeros
   scale_init: Initializer = initializers.ones
-  reduction_axes: Optional[Axes] = None
-  axis_name: Optional[str] = None
+  reduction_axes: Axes | None = None
+  axis_name: str | None = None
   axis_index_groups: Any = None
   use_fast_variance: bool = True
+  force_float32_reductions: bool = True
 
   @compact
-  def __call__(self, x, mask=None):
+  def __call__(self, x, *, mask: jax.Array | None = None):
     """Applies group normalization to the input (arxiv.org/abs/1803.08494).
 
     Args:
@@ -729,13 +789,14 @@ class GroupNorm(Module):
       mask = mask.reshape(mask.shape[:-1] + (num_groups, group_size))
 
     mean, var = _compute_stats(
-      x.reshape(group_shape),
-      list(reduction_axes[:-1]) + [-1],
-      self.dtype,
-      self.axis_name,
-      self.axis_index_groups,
-      use_fast_variance=self.use_fast_variance,
-      mask=mask,
+        x.reshape(group_shape),
+        list(reduction_axes[:-1]) + [-1],
+        self.dtype,
+        self.axis_name,
+        self.axis_index_groups,
+        use_fast_variance=self.use_fast_variance,
+        mask=mask,
+        force_float32_reductions=self.force_float32_reductions,
     )
     mean = jnp.repeat(mean, group_size, axis=-1)
     var = jnp.repeat(var, group_size, axis=-1)
@@ -754,6 +815,7 @@ class GroupNorm(Module):
       self.use_scale,
       self.bias_init,
       self.scale_init,
+      self.force_float32_reductions,
     )
 
 
@@ -767,9 +829,10 @@ class InstanceNorm(Module):
   within each channel within each example close to 0 and the activation standard
   deviation close to 1.
 
-  NOTE: This normalization operation is identical to LayerNorm and GroupNorm; the
-  difference is simply which axes are reduced and the shape of the feature axes
-  (i.e. the shape of the learnable scale and bias parameters).
+  .. note::
+    This normalization operation is identical to LayerNorm and GroupNorm; the
+    difference is simply which axes are reduced and the shape of the feature axes
+    (i.e. the shape of the learnable scale and bias parameters).
 
   Example usage::
 
@@ -816,25 +879,26 @@ class InstanceNorm(Module):
       representing subsets of devices to reduce over (default: None). For
       example, ``[[0, 1], [2, 3]]`` would independently batch-normalize over the
       examples on the first two and last two devices. See ``jax.lax.psum`` for
-      more details.
+      more details. This argument is currently not supported for SPMD jit.
     use_fast_variance: If true, use a faster, but less numerically stable,
       calculation for the variance.
   """
 
   epsilon: float = 1e-6
-  dtype: Optional[Dtype] = None
+  dtype: Dtype | None = None
   param_dtype: Dtype = jnp.float32
   use_bias: bool = True
   use_scale: bool = True
   bias_init: Initializer = initializers.zeros
   scale_init: Initializer = initializers.ones
   feature_axes: Axes = -1
-  axis_name: Optional[str] = None
+  axis_name: str | None = None
   axis_index_groups: Any = None
   use_fast_variance: bool = True
+  force_float32_reductions: bool = True
 
   @compact
-  def __call__(self, x, mask=None):
+  def __call__(self, x, *, mask: jax.Array | None = None):
     """Applies instance normalization on the input.
 
     Args:
@@ -852,13 +916,14 @@ class InstanceNorm(Module):
     reduction_axes = [i for i in range(1, x.ndim) if i not in feature_axes]
 
     mean, var = _compute_stats(
-      x,
-      reduction_axes,
-      self.dtype,
-      self.axis_name,
-      self.axis_index_groups,
-      use_fast_variance=self.use_fast_variance,
-      mask=mask,
+        x,
+        reduction_axes,
+        self.dtype,
+        self.axis_name,
+        self.axis_index_groups,
+        use_fast_variance=self.use_fast_variance,
+        mask=mask,
+        force_float32_reductions=self.force_float32_reductions,
     )
 
     return _normalize(
@@ -875,11 +940,14 @@ class InstanceNorm(Module):
       self.use_scale,
       self.bias_init,
       self.scale_init,
+      self.force_float32_reductions,
     )
 
 
 class SpectralNorm(Module):
-  """Spectral normalization. See:
+  """Spectral normalization.
+
+  See:
 
   - https://arxiv.org/abs/1802.05957
   - https://arxiv.org/abs/1805.08318
@@ -890,17 +958,17 @@ class SpectralNorm(Module):
   where each wrapped layer will have its params spectral normalized before
   computing its ``__call__`` output.
 
-  Usage Note:
-  The initialized variables dict will contain, in addition to a 'params'
-  collection, a separate 'batch_stats' collection that will contain a
-  ``u`` vector and ``sigma`` value, which are intermediate values used
-  when performing spectral normalization. During training, we pass in
-  ``update_stats=True`` and ``mutable=['batch_stats']`` so that ``u``
-  and ``sigma`` are updated with the most recently computed values using
-  power iteration. This will help the power iteration method approximate
-  the true singular value more accurately over time. During eval, we pass
-  in ``update_stats=False`` to ensure we get deterministic behavior from
-  the model. For example::
+  .. note::
+    The initialized variables dict will contain, in addition to a 'params'
+    collection, a separate 'batch_stats' collection that will contain a
+    ``u`` vector and ``sigma`` value, which are intermediate values used
+    when performing spectral normalization. During training, we pass in
+    ``update_stats=True`` and ``mutable=['batch_stats']`` so that ``u``
+    and ``sigma`` are updated with the most recently computed values using
+    power iteration. This will help the power iteration method approximate
+    the true singular value more accurately over time. During eval, we pass
+    in ``update_stats=False`` to ensure we get deterministic behavior from
+    the model.
 
   Example usage::
 
@@ -922,7 +990,7 @@ class SpectralNorm(Module):
     >>> y = jnp.ones((1, 5))
     >>> model = Foo()
     >>> variables = model.init(jax.random.PRNGKey(0), x, train=False)
-    >>> flax.core.freeze(jax.tree_map(jnp.shape, variables))
+    >>> flax.core.freeze(jax.tree_util.tree_map(jnp.shape, variables))
     FrozenDict({
         batch_stats: {
             SpectralNorm_0: {
@@ -962,7 +1030,7 @@ class SpectralNorm(Module):
     ...       variables['params']
     ...   )
     ...   return {
-    ...       'params': jax.tree_map(
+    ...       'params': jax.tree_util.tree_map(
     ...           lambda p, g: p - 0.1 * g, variables['params'], grads
     ...       ),
     ...       'batch_stats': updates['batch_stats'],
@@ -980,11 +1048,11 @@ class SpectralNorm(Module):
     epsilon: A small float added to l2-normalization to avoid dividing by zero.
     dtype: the dtype of the result (default: infer from input and params).
     param_dtype: the dtype passed to parameter initializers (default: float32).
-    error_on_non_matrix: Spectral normalization is only defined on matrices.
-      By default, this module will return scalars unchanged and flatten
+    error_on_non_matrix: Spectral normalization is only defined on matrices. By
+      default, this module will return scalars unchanged and flatten
       higher-order tensors in their leading dimensions. Setting this flag to
-      True will instead throw an error if a weight tensor with dimension
-      greater than 2 is used by the layer.
+      True will instead throw an error if a weight tensor with dimension greater
+      than 2 is used by the layer.
     collection_name: Name of the collection to store intermediate values used
       when performing spectral normalization.
   """
@@ -992,7 +1060,7 @@ class SpectralNorm(Module):
   layer_instance: Module
   n_steps: int = 1
   epsilon: float = 1e-12
-  dtype: Optional[Dtype] = None
+  dtype: Dtype | None = None
   param_dtype: Dtype = jnp.float32
   error_on_non_matrix: bool = False
   collection_name: str = 'batch_stats'
@@ -1066,7 +1134,7 @@ class SpectralNorm(Module):
     u_var_name = (
       self.layer_instance.name
       + '/'
-      + '/'.join((dict_key.key for dict_key in path[1:]))
+      + '/'.join(dict_key.key for dict_key in path[1:])
       + '/u'
     )
     u_var = self.variable(
@@ -1083,7 +1151,7 @@ class SpectralNorm(Module):
     sigma_var_name = (
       self.layer_instance.name
       + '/'
-      + '/'.join((dict_key.key for dict_key in path[1:]))
+      + '/'.join(dict_key.key for dict_key in path[1:])
       + '/sigma'
     )
     sigma_var = self.variable(
@@ -1147,8 +1215,8 @@ class WeightNorm(Module):
     ...     # l2-normalize all params of the second Dense layer
     ...     x = nn.WeightNorm(nn.Dense(4), variable_filter=None)(x)
     ...     x = nn.Dense(5)(x)
-    ...     # l2-normalize all kernels in the Bar submodule and all params in the
-    ...     # Baz submodule
+    ...     # l2-normalize all kernels in the Bar submodule and all params in
+    ...     # the Baz submodule
     ...     x = nn.WeightNorm(Bar(), variable_filter={'kernel', 'Baz'})(x)
     ...     return x
 
@@ -1156,7 +1224,7 @@ class WeightNorm(Module):
     >>> x = jnp.ones((1, 2))
     >>> model = Foo()
     >>> variables = model.init(jax.random.key(0), x)
-    >>> flax.core.freeze(jax.tree_map(jnp.shape, variables))
+    >>> flax.core.freeze(jax.tree_util.tree_map(jnp.shape, variables))
     FrozenDict({
         params: {
             Bar_0: {
@@ -1231,12 +1299,12 @@ class WeightNorm(Module):
 
   layer_instance: Module
   epsilon: float = 1e-12
-  dtype: Optional[Dtype] = None
+  dtype: Dtype | None = None
   param_dtype: Dtype = jnp.float32
   use_scale: bool = True
   scale_init: Initializer = initializers.ones
-  feature_axes: Optional[Axes] = -1
-  variable_filter: Optional[Iterable] = dataclasses.field(
+  feature_axes: Axes | None = -1
+  variable_filter: Iterable | None = dataclasses.field(
     default_factory=lambda: {'kernel'}
   )
 
@@ -1282,7 +1350,7 @@ class WeightNorm(Module):
     str_path = (
       self.layer_instance.name
       + '/'
-      + '/'.join((dict_key.key for dict_key in path[1:]))
+      + '/'.join(dict_key.key for dict_key in path[1:])
     )
     if self.variable_filter:
       for variable_name in self.variable_filter:
