@@ -1,4 +1,4 @@
-# Copyright 2022 The Flax Authors.
+# Copyright 2024 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,9 +19,8 @@ with helping write pmap-based data-parallel training loops.
 """
 
 import jax
-from jax import lax
 import jax.numpy as jnp
-import numpy as np
+from jax import lax
 
 
 def shard(xs):
@@ -35,13 +34,14 @@ def shard(xs):
   """
   local_device_count = jax.local_device_count()
   return jax.tree_util.tree_map(
-      lambda x: x.reshape((local_device_count, -1) + x.shape[1:]), xs)
+    lambda x: x.reshape((local_device_count, -1) + x.shape[1:]), xs
+  )
 
 
 def shard_prng_key(prng_key):
   """Helper to shard (aka split) a PRNGKey for use with pmap'd functions.
 
-  PRNG keys can used at train time to drive stochastic modules
+  PRNG keys can be used at train time to drive stochastic modules
   e.g. Dropout. We would like a different PRNG key for each local
   device so that we end up with different random numbers on each one,
   hence we split our PRNG key.
@@ -64,7 +64,7 @@ def stack_forest(forest):
     A single pytree of the same structure whose leaves are individually
       stacked arrays.
   """
-  stack_args = lambda *args: np.stack(args)
+  stack_args = lambda *args: jnp.stack(args)
   return jax.tree_util.tree_map(stack_args, *forest)
 
 
@@ -88,7 +88,7 @@ def get_metrics(device_metrics):
 def onehot(labels, num_classes, on_value=1.0, off_value=0.0):
   """Create a dense one-hot version of an indexed array.
 
-  NB: consider using the more standard `jax.nn.one_hot` instead.
+  NB: consider using the more standard ``jax.nn.one_hot`` instead.
 
   Args:
     labels: an n-dim JAX array whose last dimension contains integer indices.
@@ -99,6 +99,8 @@ def onehot(labels, num_classes, on_value=1.0, off_value=0.0):
     A (n+1)-dim array whose last dimension contains one-hot vectors of length
     num_classes.
   """
-  x = (labels[..., None] == jnp.arange(num_classes).reshape((1,) * labels.ndim + (-1,)))
+  x = labels[..., None] == jnp.arange(num_classes).reshape(
+    (1,) * labels.ndim + (-1,)
+  )
   x = lax.select(x, jnp.full(x.shape, on_value), jnp.full(x.shape, off_value))
   return x.astype(jnp.float32)
