@@ -1,4 +1,4 @@
-# Copyright 2022 The Flax Authors.
+# Copyright 2024 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
 """Early stopping."""
 
 import math
+
 from flax import struct
 
 
@@ -25,15 +26,23 @@ class EarlyStopping(struct.PyTreeNode):
   recorded in the current epoch and previous epoch is less than 1e-3
   consecutively for 2 times::
 
-    early_stop = EarlyStopping(min_delta=1e-3, patience=2)
-    for epoch in range(1, num_epochs+1):
-      rng, input_rng = jax.random.split(rng)
-      optimizer, train_metrics = train_epoch(
-          optimizer, train_ds, config.batch_size, epoch, input_rng)
-      _, early_stop = early_stop.update(train_metrics['loss'])
-      if early_stop.should_stop:
-        print('Met early stopping criteria, breaking...')
-        break
+    >>> from flax.training.early_stopping import EarlyStopping
+
+    >>> def train_epoch(optimizer, train_ds, batch_size, epoch, input_rng):
+    ...   ...
+    ...   loss = [4, 3, 3, 3, 2, 2, 2, 2, 1, 1][epoch]
+    ...   return None, {'loss': loss}
+
+    >>> early_stop = EarlyStopping(min_delta=1e-3, patience=2)
+    >>> optimizer = None
+    >>> for epoch in range(10):
+    ...   optimizer, train_metrics = train_epoch(
+    ...       optimizer=optimizer, train_ds=None, batch_size=None, epoch=epoch, input_rng=None)
+    ...   early_stop = early_stop.update(train_metrics['loss'])
+    ...   if early_stop.should_stop:
+    ...     print(f'Met early stopping criteria, breaking at epoch {epoch}')
+    ...     break
+    Met early stopping criteria, breaking at epoch 7
 
   Attributes:
     min_delta: Minimum delta between updates to be considered an
@@ -43,31 +52,44 @@ class EarlyStopping(struct.PyTreeNode):
     patience_count: Number of steps since last improving update.
     should_stop: Whether the training loop should stop to avoid
         overfitting.
+    has_improved: Whether the metric has improved greater or
+      equal to the min_delta in the last ``.update`` call.
   """
+
   min_delta: float = 0
   patience: int = 0
   best_metric: float = float('inf')
   patience_count: int = 0
   should_stop: bool = False
+  has_improved: bool = False
 
   def reset(self):
-    return self.replace(best_metric=float('inf'),
-                        patience_count=0,
-                        should_stop=False)
+    return self.replace(
+      best_metric=float('inf'),
+      patience_count=0,
+      should_stop=False,
+      has_improved=False,
+    )
 
   def update(self, metric):
     """Update the state based on metric.
 
     Returns:
-      A pair (has_improved, early_stop), where `has_improved` is True when there
-      was an improvement greater than `min_delta` from the previous
-      `best_metric` and `early_stop` is the updated `EarlyStop` object.
+      The updated EarlyStopping class. The ``.has_improved`` attribute is True
+      when there was an improvement greater than ``min_delta`` from the previous
+      ``best_metric``.
     """
 
-    if math.isinf(self.best_metric) or self.best_metric - metric > self.min_delta:
-      return True, self.replace(best_metric=metric,
-                                patience_count=0)
+    if (
+      math.isinf(self.best_metric) or self.best_metric - metric > self.min_delta
+    ):
+      return self.replace(
+        best_metric=metric, patience_count=0, has_improved=True
+      )
     else:
       should_stop = self.patience_count >= self.patience or self.should_stop
-      return False, self.replace(patience_count=self.patience_count + 1,
-                                 should_stop=should_stop)
+      return self.replace(
+        patience_count=self.patience_count + 1,
+        should_stop=should_stop,
+        has_improved=False,
+      )
