@@ -1,4 +1,4 @@
-# Copyright 2022 The Flax Authors.
+# Copyright 2024 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,9 +23,9 @@ Note that with `FLIP #1009`_ learning rate schedules in ``flax.training`` are
 .. _Optimizer Schedules: https://optax.readthedocs.io/en/latest/api.html#optimizer-schedules
 """
 
-from absl import logging
 import jax.numpy as jnp
 import numpy as np
+from absl import logging
 
 
 def _piecewise_constant(boundaries, values, t):
@@ -33,8 +33,9 @@ def _piecewise_constant(boundaries, values, t):
   return jnp.take(values, index)
 
 
-def create_constant_learning_rate_schedule(base_learning_rate, steps_per_epoch,
-                                           warmup_length=0.0):
+def create_constant_learning_rate_schedule(
+  base_learning_rate, steps_per_epoch, warmup_length=0.0
+):
   """Create a constant learning rate schedule with optional warmup.
 
   Note that with `FLIP #1009`_ learning rate schedules in ``flax.training`` are
@@ -54,26 +55,30 @@ def create_constant_learning_rate_schedule(base_learning_rate, steps_per_epoch,
     steps_per_epoch: the number of iterations per epoch
     warmup_length: if > 0, the learning rate will be modulated by a warmup
       factor that will linearly ramp-up from 0 to 1 over the first
-      `warmup_length` epochs
+      ``warmup_length`` epochs
 
   Returns:
-    Function `f(step) -> lr` that computes the learning rate for a given step.
+    Function ``f(step) -> lr`` that computes the learning rate for a given step.
   """
   logging.warning(
     'Learning rate schedules in ``flax.training`` are effectively deprecated '
     'in favor of Optax schedules. Please refer to '
     'https://optax.readthedocs.io/en/latest/api.html#optimizer-schedules'
-    ' for alternatives.')
+    ' for alternatives.'
+  )
+
   def learning_rate_fn(step):
     lr = base_learning_rate
     if warmup_length > 0.0:
-      lr = lr * jnp.minimum(1., step / float(warmup_length) / steps_per_epoch)
+      lr = lr * jnp.minimum(1.0, step / float(warmup_length) / steps_per_epoch)
     return lr
+
   return learning_rate_fn
 
 
-def create_stepped_learning_rate_schedule(base_learning_rate, steps_per_epoch,
-                                          lr_sched_steps, warmup_length=0.0):
+def create_stepped_learning_rate_schedule(
+  base_learning_rate, steps_per_epoch, lr_sched_steps, warmup_length=0.0
+):
   """Create a stepped learning rate schedule with optional warmup.
 
   Note that with `FLIP #1009`_ learning rate schedules in ``flax.training`` are
@@ -86,7 +91,7 @@ def create_stepped_learning_rate_schedule(base_learning_rate, steps_per_epoch,
 
   A stepped learning rate schedule decreases the learning rate
   by specified amounts at specified epochs. The steps are given as
-  the `lr_sched_steps` parameter. A common ImageNet schedule decays the
+  the ``lr_sched_steps`` parameter. A common ImageNet schedule decays the
   learning rate by a factor of 0.1 at epochs 30, 60 and 80. This would be
   specified as::
 
@@ -104,20 +109,21 @@ def create_stepped_learning_rate_schedule(base_learning_rate, steps_per_epoch,
     base_learning_rate: the base learning rate
     steps_per_epoch: the number of iterations per epoch
     lr_sched_steps: the schedule as a list of steps, each of which is
-      a `[epoch, lr_factor]` pair; the step occurs at epoch `epoch` and
-      sets the learning rate to `base_learning_rage * lr_factor`
+      a ``[epoch, lr_factor]`` pair; the step occurs at epoch ``epoch`` and
+      sets the learning rate to ``base_learning_rage * lr_factor``
     warmup_length: if > 0, the learning rate will be modulated by a warmup
       factor that will linearly ramp-up from 0 to 1 over the first
-      `warmup_length` epochs
+      ``warmup_length`` epochs
 
   Returns:
-    Function `f(step) -> lr` that computes the learning rate for a given step.
+    Function ``f(step) -> lr`` that computes the learning rate for a given step.
   """
   logging.warning(
     'Learning rate schedules in ``flax.training`` are effectively deprecated '
     'in favor of Optax schedules. Please refer to '
     'https://optax.readthedocs.io/en/latest/api.html#optimizer-schedules'
-    ' for alternatives.')
+    ' for alternatives.'
+  )
   boundaries = [step[0] for step in lr_sched_steps]
   decays = [step[1] for step in lr_sched_steps]
   boundaries = np.array(boundaries) * steps_per_epoch
@@ -127,13 +133,15 @@ def create_stepped_learning_rate_schedule(base_learning_rate, steps_per_epoch,
   def learning_rate_fn(step):
     lr = _piecewise_constant(boundaries, values, step)
     if warmup_length > 0.0:
-      lr = lr * jnp.minimum(1., step / float(warmup_length) / steps_per_epoch)
+      lr = lr * jnp.minimum(1.0, step / float(warmup_length) / steps_per_epoch)
     return lr
+
   return learning_rate_fn
 
 
-def create_cosine_learning_rate_schedule(base_learning_rate, steps_per_epoch,
-                                         halfcos_epochs, warmup_length=0.0):
+def create_cosine_learning_rate_schedule(
+  base_learning_rate, steps_per_epoch, halfcos_epochs, warmup_length=0.0
+):
   """Create a cosine learning rate schedule with optional warmup.
 
   Note that with `FLIP #1009`_ learning rate schedules in ``flax.training`` are
@@ -158,24 +166,24 @@ def create_cosine_learning_rate_schedule(base_learning_rate, steps_per_epoch,
       normally the number of epochs used for training
     warmup_length: if > 0, the learning rate will be modulated by a warmup
       factor that will linearly ramp-up from 0 to 1 over the first
-      `warmup_length` epochs
+      ``warmup_length`` epochs
 
   Returns:
-    Function `f(step) -> lr` that computes the learning rate for a given step.
+    Function ``f(step) -> lr`` that computes the learning rate for a given step.
   """
   logging.warning(
     'Learning rate schedules in ``flax.training`` are effectively deprecated '
     'in favor of Optax schedules. Please refer to '
     'https://optax.readthedocs.io/en/latest/api.html#optimizer-schedules'
-    ' for alternatives.')
+    ' for alternatives.'
+  )
   halfwavelength_steps = halfcos_epochs * steps_per_epoch
 
   def learning_rate_fn(step):
     scale_factor = jnp.cos(step * jnp.pi / halfwavelength_steps) * 0.5 + 0.5
     lr = base_learning_rate * scale_factor
     if warmup_length > 0.0:
-      lr = lr * jnp.minimum(1., step / float(warmup_length) / steps_per_epoch)
+      lr = lr * jnp.minimum(1.0, step / float(warmup_length) / steps_per_epoch)
     return lr
+
   return learning_rate_fn
-
-
