@@ -14,16 +14,21 @@
 
 """Tests for flax.linen.attention."""
 
-import jax
-import jax.numpy as jnp
-import numpy as np
 from absl.testing import absltest, parameterized
-from jax import lax, random
-from jax.nn import initializers
-
 from flax import errors, jax_utils
 from flax import linen as nn
 from flax.core import pop
+import jax
+from jax import lax, random
+from jax.nn import initializers
+import jax.numpy as jnp
+import numpy as np
+
+try:
+  # JAX v0.8.0 and newer
+  from jax import enable_x64
+except ImportError:
+  from jax.experimental import enable_x64
 
 # Parse absl flags test_srcdir and test_tmpdir.
 jax.config.parse_flags_with_absl()
@@ -71,6 +76,31 @@ class AttentionTest(parameterized.TestCase):
     y, _ = sa_module.init_with_output(rng, q)
     self.assertEqual(y.shape, q.shape)
 
+  def test_mha_out_initializers(self):
+    rng = random.key(0)
+    q = jnp.ones((4, 2, 3, 5))
+    sa_module = nn.MultiHeadDotProductAttention(
+      num_heads=8,
+      qkv_features=16,
+      kernel_init=initializers.ones,
+      out_kernel_init=initializers.zeros,
+      bias_init=initializers.zeros,
+      out_bias_init=initializers.ones,
+      deterministic=False,
+    )
+    variables = sa_module.init(rng, q)
+    params = variables['params']
+    # test kernels
+    np.testing.assert_allclose(params['query']['kernel'], 1.0)
+    np.testing.assert_allclose(params['key']['kernel'], 1.0)
+    np.testing.assert_allclose(params['value']['kernel'], 1.0)
+    np.testing.assert_allclose(params['out']['kernel'], 0.0)
+    # test biases
+    np.testing.assert_allclose(params['query']['bias'], 0.0)
+    np.testing.assert_allclose(params['key']['bias'], 0.0)
+    np.testing.assert_allclose(params['value']['bias'], 0.0)
+    np.testing.assert_allclose(params['out']['bias'], 1.0)
+
   def test_multihead_self_attention_w_dropout(self):
     rng = random.key(0)
     x = jnp.ones((4, 2, 3, 5))
@@ -88,6 +118,9 @@ class AttentionTest(parameterized.TestCase):
     self.assertEqual(y.shape, x.shape)
 
   def test_multihead_self_attention_explicit_dropout(self):
+    def clone(key):
+      return jax.tree.map(jax.random.clone, key)
+
     class Foo(nn.Module):
       attention_kwargs: dict
 
@@ -96,6 +129,8 @@ class AttentionTest(parameterized.TestCase):
         a = nn.MultiHeadDotProductAttention(**self.attention_kwargs)(
           x, x, dropout_rng=dropout_rng
         )
+        if dropout_rng is not None:
+          dropout_rng = clone(dropout_rng)
         b = nn.MultiHeadDotProductAttention(**self.attention_kwargs)(
           x, x, dropout_rng=dropout_rng
         )
@@ -115,17 +150,17 @@ class AttentionTest(parameterized.TestCase):
     x = jnp.ones((4, 2, 3, 5))
     rngs = {'params': rng1, 'dropout': rng2}
     v = module.init(rngs, x)
-    a, b = module.apply(v, x, rngs=rngs)
-    c, d = module.apply(v, x, rngs={'dropout': rng2})
+    a, b = module.apply(v, x, rngs=clone(rngs))
+    c, d = module.apply(v, x, rngs={'dropout': clone(rng2)})
     e, f = module.apply(v, x, rngs={'dropout': rng3})
-    self.assertTrue(not (a == b).all())
+    self.assertFalse((a == b).all())
     self.assertTrue((a == c).all())
     self.assertTrue((b == d).all())
-    self.assertTrue(not (a == e).all())
-    self.assertTrue(not (b == f).all())
-    a, b = module.apply(v, x, rngs=rngs, dropout_rng=rng4)
+    self.assertFalse((a == e).all())
+    self.assertFalse((b == f).all())
+    a, b = module.apply(v, x, rngs=clone(rngs), dropout_rng=rng4)
     self.assertTrue((a == b).all())
-    a, b = module.apply(v, x, dropout_rng=rng4)
+    a, b = module.apply(v, x, dropout_rng=clone(rng4))
     self.assertTrue((a == b).all())
     self.assertTrue(a.shape == b.shape == x.shape)
 
@@ -272,7 +307,7 @@ class AttentionTest(parameterized.TestCase):
   def test_multihead_kv_args(self):
     key1, key2 = random.split(random.key(0), 2)
     query = random.uniform(key1, (3, 5))
-    key_value = random.uniform(key1, (9, 5))
+    key_value = random.uniform(key2, (9, 5))
     module = nn.MultiHeadDotProductAttention(
       num_heads=8,
       qkv_features=16,
@@ -280,40 +315,41 @@ class AttentionTest(parameterized.TestCase):
       bias_init=initializers.zeros,
       deterministic=False,
     )
+    key = lambda: random.key(43279)
     y0, v0 = module.init_with_output(
-      key2, query, inputs_k=key_value, inputs_v=key_value
+      key(), query, inputs_k=key_value, inputs_v=key_value
     )
-    y1, v1 = module.init_with_output(key2, query, inputs_k=key_value)
+    y1, v1 = module.init_with_output(key(), query, inputs_k=key_value)
     with self.assertWarnsRegex(
       DeprecationWarning, 'The inputs_kv arg will be deprecated soon.'
     ):
-      y2, v2 = module.init_with_output(key2, query, inputs_kv=key_value)
+      y2, v2 = module.init_with_output(key(), query, inputs_kv=key_value)
     self.assertTrue((y0 == y1).all() and (y1 == y2).all())
     self.assertTrue(
-      jax.tree_util.tree_all(
-        jax.tree_map(
-          lambda x, y, z: (x == y).all() and (y == z).all(), v0, v1, v2
+        jax.tree_util.tree_all(
+            jax.tree_util.tree_map(
+                lambda x, y, z: (x == y).all() and (y == z).all(), v0, v1, v2
+            )
         )
-      )
     )
 
     with self.assertRaisesRegex(
       ValueError, '`inputs_k` cannot be None if `inputs_v` is not None.'
     ):
-      y3, v3 = module.init_with_output(key2, query, inputs_v=key_value)
+      y3, v3 = module.init_with_output(key(), query, inputs_v=key_value)
     with self.assertRaisesRegex(
       ValueError,
       'If either `inputs_k` or `inputs_v` is not None, `inputs_kv` must be None.',
     ):
       y3, v3 = module.init_with_output(
-        key2, query, inputs_kv=key_value, inputs_v=key_value
+        key(), query, inputs_kv=key_value, inputs_v=key_value
       )
     with self.assertRaisesRegex(
       ValueError,
       'If either `inputs_k` or `inputs_v` is not None, `inputs_kv` must be None.',
     ):
       y3, v3 = module.init_with_output(
-        key2, query, key_value, key_value, inputs_kv=key_value
+        key(), query, key_value, key_value, inputs_kv=key_value
       )
 
   def test_multihead_mask_warning(self):
@@ -395,35 +431,35 @@ class AttentionTest(parameterized.TestCase):
     self.assertNotIn('intermediates', intermediates)
 
   def test_autoregressive_decode_with_x64(self):
-    with jax.experimental.enable_x64():
-        x = jnp.ones((1, 4, 4))
-        module = nn.MultiHeadDotProductAttention(
+    with enable_x64():
+      x = jnp.ones((1, 4, 4))
+      module = nn.MultiHeadDotProductAttention(
           num_heads=2,
           qkv_features=4,
           decode=True
         )
 
-        rng = random.PRNGKey(0)
-        variables = module.init(rng, x, x, x)
-        params, cache = variables['params'], variables['cache']
-        y1, updates = module.apply(
+      rng = random.PRNGKey(0)
+      variables = module.init(rng, x, x, x)
+      params, cache = variables['params'], variables['cache']
+      y1, updates = module.apply(
           { 'params': params, 'cache': cache },
           x[:, :1, :],
           mutable=['cache']
         )
-        cache = updates['cache']
-        y2, updates = module.apply(
+      cache = updates['cache']
+      y2, updates = module.apply(
           { 'params': params, 'cache': cache },
           x[:, 1:2, :],
           mutable=['cache']
         )
-        assert y1.shape == (1, 1, 4)
-        assert y2.shape == (1, 1, 4)
+      assert y1.shape == (1, 1, 4)
+      assert y2.shape == (1, 1, 4)
 
   def test_attention_alias_equivalence(self):
     key1, key2 = random.split(random.key(0), 2)
     query = random.uniform(key1, (3, 5))
-    key_value = random.uniform(key1, (9, 5))
+    key_value = random.uniform(key2, (9, 5))
     attention_kwargs = dict(
       num_heads=8,
       qkv_features=16,
@@ -433,17 +469,20 @@ class AttentionTest(parameterized.TestCase):
     )
     module1 = nn.MultiHeadDotProductAttention(**attention_kwargs)
     module2 = nn.MultiHeadAttention(**attention_kwargs)
-    out1, v1 = module1.init_with_output(key2, query, key_value)
-    out2, v2 = module2.init_with_output(key2, query, key_value, key_value)
+    key = lambda: random.key(43279)
+    out1, v1 = module1.init_with_output(key(), query, key_value)
+    out2, v2 = module2.init_with_output(key(), query, key_value, key_value)
     self.assertTrue((out1 == out2).all())
     self.assertTrue(
-      jax.tree_util.tree_all(jax.tree_map(lambda x, y: (x == y).all(), v1, v2))
+        jax.tree_util.tree_all(
+            jax.tree_util.tree_map(lambda x, y: (x == y).all(), v1, v2)
+        )
     )
 
   def test_attention_alias_submodule(self):
     key1, key2 = random.split(random.key(0), 2)
     query = random.uniform(key1, (3, 5))
-    key_value = random.uniform(key1, (9, 5))
+    key_value = random.uniform(key2, (9, 5))
     attention_kwargs = dict(
       num_heads=8,
       qkv_features=16,
@@ -468,10 +507,11 @@ class AttentionTest(parameterized.TestCase):
       def __call__(self, query, key, value):
         return nn.MultiHeadAttention(**self.attention_kwargs)(query, key, value)
 
+    key = lambda: random.key(5478392)
     module1 = Foo1(attention_kwargs)
     module2 = Foo2(attention_kwargs)
-    out1, v1 = module1.init_with_output(key2, query, key_value)
-    out2, v2 = module2.init_with_output(key2, query, key_value, key_value)
+    out1, v1 = module1.init_with_output(key(), query, key_value)
+    out2, v2 = module2.init_with_output(key(), query, key_value, key_value)
 
     # test different output and variables if layer names are different
     self.assertTrue((out1 != out2).all())
@@ -480,11 +520,13 @@ class AttentionTest(parameterized.TestCase):
     ]
     del v2['params']['MultiHeadAttention_0']
     self.assertTrue(
-      jax.tree_util.tree_all(jax.tree_map(lambda x, y: (x != y).all(), v1, v2))
+        jax.tree_util.tree_all(
+            jax.tree_util.tree_map(lambda x, y: (x != y).all(), v1, v2)
+        )
     )
 
     # test same output if variables are the same
-    v2 = jax.tree_map(lambda x: x, v1)
+    v2 = jax.tree_util.tree_map(lambda x: x, v1)
     v2['params']['MultiHeadAttention_0'] = v2['params'][
       'MultiHeadDotProductAttention_0'
     ]
@@ -503,10 +545,81 @@ class AttentionTest(parameterized.TestCase):
         )(query, key, value)
 
     module2 = Foo2(attention_kwargs)
-    out2, v2 = module2.init_with_output(key2, query, key_value, key_value)
+    out2, v2 = module2.init_with_output(key(), query, key_value, key_value)
     self.assertTrue((out1 == out2).all())
     self.assertTrue(
-      jax.tree_util.tree_all(jax.tree_map(lambda x, y: (x == y).all(), v1, v2))
+        jax.tree_util.tree_all(
+            jax.tree_util.tree_map(lambda x, y: (x == y).all(), v1, v2)
+        )
+    )
+
+  @parameterized.parameters(
+      {'force_fp32': True, 'attn_weights_dtype': jnp.float32},
+      {'force_fp32': False, 'attn_weights_dtype': jnp.bfloat16},
+  )
+  def test_mixed_precision_multihead_attention(
+      self, force_fp32, attn_weights_dtype
+  ):
+    input_key, params_key, dropout_key = random.split(random.key(0), 3)
+    x = random.uniform(input_key, (2, 4))
+    attention_kwargs = dict(
+        num_heads=2,
+        qkv_features=4,
+        kernel_init=initializers.lecun_normal(),
+        bias_init=initializers.uniform(),
+        force_fp32_for_softmax=force_fp32,
+        deterministic=False,
+        dtype=jnp.bfloat16,
+    )
+    mha = nn.MultiHeadDotProductAttention(**attention_kwargs)
+    init_vars = mha.init({'params': params_key, 'dropout': dropout_key}, x)
+    _, updated_vars = mha.apply(
+        init_vars, x, mutable=['intermediates'], sow_weights=True
+    )
+    self.assertEqual(
+        updated_vars['intermediates']['attention_weights'][0].dtype,
+        attn_weights_dtype,
+    )
+
+  @parameterized.parameters(
+      (lax.Precision.DEFAULT, None),
+      (None, jax.lax.dot_general),
+  )
+  def test_dot_product_attention_precision_and_einsum_override(
+      self, precision, einsum_dot_general
+  ):
+    # Test that we raise a ValueError if the user specifies both
+    # `precision` and/or `einsum_dot_general` and `qk_attn_weights_einsum`.
+    einsum_cls = lambda: jnp.einsum
+    self.assertRaises(
+        ValueError,
+        nn.dot_product_attention,
+        query=jnp.ones((1, 4, 2)),
+        key=jnp.ones((1, 4, 2)),
+        value=jnp.ones((1, 4, 2)),
+        precision=precision,
+        einsum_dot_general=einsum_dot_general,
+        qk_attn_weights_einsum=einsum_cls,
+        attn_weights_value_einsum=einsum_cls,
+    )
+
+  @parameterized.parameters(
+      (lambda: jax.lax.dot_general, None),
+      (None, lambda: jax.lax.dot_general),
+  )
+  def test_dot_product_attention_specify_einsums_together(
+      self, qk_attn_weights_einsum, attn_weights_value_einsum
+  ):
+    # Test that we raise a ValueError if the user specifies only one of
+    # `qk_attn_weights_einsum` and `attn_weights_value_einsum`.
+    self.assertRaises(
+        ValueError,
+        nn.dot_product_attention,
+        query=jnp.ones((1, 4, 2)),
+        key=jnp.ones((1, 4, 2)),
+        value=jnp.ones((1, 4, 2)),
+        qk_attn_weights_einsum=qk_attn_weights_einsum,
+        attn_weights_value_einsum=attn_weights_value_einsum,
     )
 
 
