@@ -181,6 +181,246 @@ class LinearTest(parameterized.TestCase):
     y = dense.apply(variables, x)
     self.assertEqual(y.dtype, jnp.complex64)
 
+  @parameterized.parameters(
+    [
+      (
+        'abc,cde->abde',
+        (3, 4, 5),
+        (5, 6, 7),
+        (3, 4, 6, 7),
+        (6, 7),
+        (1, 1, 6, 7),
+      ),
+      (
+        'abcd,abcd->abcd',
+        (3, 4, 5, 6),
+        (3, 4, 5, 6),
+        (3, 4, 5, 6),
+        (3, 4, 5, 6),
+        (3, 4, 5, 6),
+      ),
+      (
+        'abcd,abcd->abd',
+        (3, 4, 5, 6),
+        (3, 4, 5, 6),
+        (3, 4, 6),
+        (3, 4, 6),
+        (3, 4, 6),
+      ),
+      (
+        'abcd,cdef->abef',
+        (3, 4, 5, 6),
+        (5, 6, 7, 8),
+        (3, 4, 7, 8),
+        (7, 8),
+        (1, 1, 7, 8),
+      ),
+      (
+        'abcd,eafc->bfed',
+        (3, 4, 5, 6),
+        (7, 3, 8, 5),
+        (4, 8, 7, 6),
+        (8, 7),
+        (1, 8, 7, 1),
+      ),
+      (
+        'abcd,cbedf->abfe',
+        (3, 4, 5, 6),
+        (5, 4, 7, 6, 8),
+        (3, 4, 8, 7),
+        (4, 8, 7),
+        (1, 4, 8, 7),
+      ),
+      (
+        'ab...,bc...->ac...',
+        (3, 4, 6),
+        (4, 5, 6),
+        (3, 5, 6),
+        (5, 6),
+        (1, 5, 6),
+      ),
+      (
+        'd...ab,bc...->ad...c',
+        (8, 6, 7, 3, 4),
+        (4, 5, 6, 7),
+        (3, 8, 6, 7, 5),
+        (6, 7, 5),
+        (1, 1, 6, 7, 5),
+      ),
+      (
+        'd...ab,bc...->adc',
+        (8, 6, 7, 3, 4),
+        (4, 5, 6, 7),
+        (3, 8, 5),
+        (5,),
+        (1, 1, 5),
+      ),
+      (
+        'abd...,bc...->ac...',
+        (3, 4, 6),
+        (4, 5, 6),
+        (3, 5, 6),
+        (5, 6),
+        (1, 5, 6),
+      ),
+      (
+        'a...d,ej...f->adef',
+        (3, 4, 5, 6),
+        (7, 4, 5, 8),
+        (3, 6, 7, 8),
+        (7, 8),
+        (1, 1, 7, 8),
+      ),
+      (
+        'ab...d,ej...f->ad...f',
+        (3, 4, 5, 6),
+        (7, 4, 5, 8),
+        (3, 6, 5, 8),
+        (5, 8),
+        (1, 1, 5, 8),
+      ),
+    ]
+  )
+  def test_einsum_init_apply(
+    self,
+    einsum_str,
+    lhs_shape,
+    rhs_shape,
+    expected_result_shape,
+    expected_bias_shape,
+    bias_broadcast_shape,
+  ):
+    layer = nn.Einsum(rhs_shape, einsum_str, bias_init=nn.initializers.normal())
+    x = jax.random.normal(jax.random.key(0), lhs_shape)
+
+    v = layer.init(jax.random.key(1), x)
+    self.assertEqual(rhs_shape, v['params']['kernel'].shape)
+    self.assertEqual(expected_bias_shape, v['params']['bias'].shape)
+
+    out = layer.apply(v, x)
+    self.assertEqual(out.shape, expected_result_shape)
+    expected_out = jnp.einsum(einsum_str, x, v['params']['kernel']) + v[
+      'params'
+    ]['bias'].reshape(bias_broadcast_shape)
+    np.testing.assert_allclose(out, expected_out)
+
+  @parameterized.parameters(
+    [
+      (
+        ('abd,bce->ace', 'abd,bc...->ac...', 'abd...,bc...->ac...'),
+        (3, 4, 6),
+        (4, 5, 6),
+      ),
+      (
+        (
+          'abcd,ejcf->adef',
+          'ab...d,ej...f->adef',
+          'ab...d,e...f->adef',
+          'a...d,ej...f->adef',
+        ),
+        (3, 4, 5, 6),
+        (7, 4, 5, 8),
+      ),
+      (
+        ('abcd,ejcf->adcf', 'ab...d,ej...f->ad...f'),
+        (3, 4, 5, 6),
+        (7, 4, 5, 8),
+      ),
+    ]
+  )
+  def test_einsum_ellipsis_equivalence(
+    self, einsum_str_list, lhs_shape, rhs_shape
+  ):
+    x = jax.random.uniform(jax.random.key(0), lhs_shape)
+    layer = nn.Einsum(
+      rhs_shape, einsum_str_list[0], bias_init=nn.initializers.normal()
+    )
+    v = layer.init(jax.random.key(1), x)
+    out = layer.apply(v, x)
+
+    for einsum_str in einsum_str_list[1:]:
+      layer2 = nn.Einsum(
+        rhs_shape, einsum_str, bias_init=nn.initializers.normal()
+      )
+      v2 = layer2.init(jax.random.key(1), x)
+      np.testing.assert_allclose(v['params']['kernel'], v2['params']['kernel'])
+      np.testing.assert_allclose(v['params']['bias'], v2['params']['bias'])
+      np.testing.assert_allclose(out, layer2.apply(v2, x))
+
+  def test_einsum_str_arg(self):
+    einsum_str = 'abc,cde->abde'
+    x = jax.random.normal(jax.random.key(0), (3, 4, 5))
+
+    constructed_layer = nn.Einsum(
+      (5, 6, 7), einsum_str, bias_init=nn.initializers.normal()
+    )
+    constructed_v = constructed_layer.init(jax.random.key(1), x)
+    constructed_out = constructed_layer.apply(constructed_v, x)
+
+    called_layer = nn.Einsum((5, 6, 7), bias_init=nn.initializers.normal())
+    called_v = called_layer.init(jax.random.key(1), x, einsum_str)
+    called_out = called_layer.apply(called_v, x, einsum_str)
+
+    np.testing.assert_allclose(
+      constructed_v['params']['kernel'], called_v['params']['kernel']
+    )
+    np.testing.assert_allclose(
+      constructed_v['params']['bias'], called_v['params']['bias']
+    )
+    np.testing.assert_allclose(constructed_out, called_out)
+
+    with self.assertRaisesWithLiteralMatch(
+      ValueError,
+      'Parameter "einsum_str" was passed to the constructor and at call time. Should be passed just once.',
+    ):
+      constructed_layer.init(jax.random.key(1), x, einsum_str)
+    with self.assertRaisesWithLiteralMatch(
+      ValueError,
+      'Parameter "einsum_str" must be passed to the constructor or at call time.',
+    ):
+      called_layer.init(jax.random.key(1), x)
+
+  def test_einsum_space_str(self):
+    x = jax.random.normal(jax.random.key(0), (3, 4, 5))
+
+    layer1 = nn.Einsum(
+      (5, 6, 7), 'abc,cde->abde', bias_init=nn.initializers.normal()
+    )
+    v1 = layer1.init(jax.random.key(1), x)
+    out1 = layer1.apply(v1, x)
+
+    layer2 = nn.Einsum(
+      (5, 6, 7),
+      '  ab c  , c d e    -  >a b d  e  ',
+      bias_init=nn.initializers.normal(),
+    )
+    v2 = layer2.init(jax.random.key(1), x)
+    out2 = layer2.apply(v1, x)
+
+    np.testing.assert_allclose(v1['params']['kernel'], v2['params']['kernel'])
+    np.testing.assert_allclose(v1['params']['bias'], v2['params']['bias'])
+    np.testing.assert_allclose(out1, out2)
+
+  @parameterized.parameters(
+    [
+      ('abc,cde', '`einsum_str` equation must be explicit and include "->".'),
+      (
+        'abc->',
+        f'`einsum_str` equation must have exactly two operands and therefore, exactly one comma character, instead of 0',
+      ),
+      (
+        'abc,cde,efg->abdfg',
+        f'`einsum_str` equation must have exactly two operands and therefore, exactly one comma character, instead of 2',
+      ),
+    ]
+  )
+  def test_einsum_error(self, einsum_str, error_msg):
+    x = jax.random.normal(jax.random.key(0), (3, 4, 5))
+
+    layer = nn.Einsum((5, 6, 7), einsum_str)
+    with self.assertRaisesRegex(ValueError, error_msg):
+      layer.init(jax.random.key(1), x)
+
   @parameterized.product(use_bias=(True, False))
   def test_conv(self, use_bias):
     rng = dict(params=random.key(0))
@@ -606,6 +846,59 @@ class LinearTest(parameterized.TestCase):
     correct_ans = np.expand_dims(correct_ans, (0, 3))
     np.testing.assert_allclose(y, correct_ans)
 
+  def test_reflect_conv_1d_custom(self):
+    """Test 1d convolution with reflection padding and a stride."""
+    rng = dict(params=random.key(0))
+    x = np.arange(1, 6)
+    x = np.expand_dims(x, (0, 2))
+    kernel = np.array((1, 2, 1))
+    kernel = np.expand_dims(kernel, (1, 2))
+
+    conv_module = nn.Conv(
+      features=1,
+      kernel_size=(3,),
+      strides=(2,),
+      padding='REFLECT',
+      kernel_init=lambda *_: kernel,
+      bias_init=initializers.zeros,
+    )
+    y, initial_params = conv_module.init_with_output(rng, x)
+
+    self.assertEqual(initial_params['params']['kernel'].shape, (3, 1, 1))
+    # Compare with manually computed convolution
+    correct_ans = np.array((2 + 2 * 1 + 2, 2 + 2 * 3 + 4, 4 + 2 * 5 + 4))
+    correct_ans = np.expand_dims(correct_ans, (0, 2))
+    np.testing.assert_allclose(y, correct_ans)
+
+  def test_reflect_conv_2d_custom(self):
+    """Test 2d convolution with reflect padding on a 3x3 example."""
+    rng = dict(params=random.key(0))
+    x = np.array(((1, 2, 3), (4, 5, 6), (7, 8, 9)))
+    x = np.expand_dims(x, (0, 3))
+    kernel = np.array(((0, 1, 0), (1, 2, 1), (0, 1, 0)))
+    kernel = np.expand_dims(kernel, (2, 3))
+
+    conv_module = nn.Conv(
+      features=1,
+      kernel_size=(3, 3),
+      padding='REFLECT',
+      kernel_init=lambda *_: kernel,
+      bias_init=initializers.zeros,
+    )
+    y, initial_params = conv_module.init_with_output(rng, x)
+
+    self.assertEqual(initial_params['params']['kernel'].shape, (3, 3, 1, 1))
+    # Compare with manually computed convolution
+    correct_ans = np.array(
+      (
+        (2 * 1 + 4 + 2 + 4 + 2, 2 * 2 + 5 + 3 + 5 + 1, 2 * 3 + 6 + 2 + 6 + 2),
+        (2 * 4 + 1 + 5 + 7 + 5, 2 * 5 + 2 + 6 + 8 + 4, 2 * 6 + 3 + 5 + 9 + 5),
+        (2 * 7 + 4 + 8 + 8 + 4, 2 * 8 + 5 + 9 + 5 + 7, 2 * 9 + 6 + 8 + 6 + 8),
+      )
+    )
+    correct_ans = np.expand_dims(correct_ans, (0, 3))
+    np.testing.assert_allclose(y, correct_ans)
+
   def test_causal_conv1d(self):
     rng = dict(params=random.key(0))
     x = jnp.ones((1, 8, 4))
@@ -980,7 +1273,7 @@ class LinearTest(parameterized.TestCase):
     conv_module = nn.ConvTranspose(
       features=4,
       use_bias=use_bias,
-      strides=(2, 2),
+      strides=2,
       kernel_size=(6, 6),
       padding='CIRCULAR',
       transpose_kernel=True,
@@ -995,13 +1288,13 @@ class LinearTest(parameterized.TestCase):
     conv_seq = module(features=4, kernel_size=(3,))
     x = jnp.ones((8, 3))
     self.assertTrue(
-      jax.tree_util.tree_all(
-        jax.tree_map(
-          lambda x, y: (x == y).all(),
-          conv_int.init(random.key(0), x),
-          conv_seq.init(random.key(0), x),
+        jax.tree_util.tree_all(
+            jax.tree_util.tree_map(
+                lambda x, y: (x == y).all(),
+                conv_int.init(random.key(0), x),
+                conv_seq.init(random.key(0), x),
+            )
         )
-      )
     )
 
   def test_embed(self):
@@ -1040,40 +1333,6 @@ class LinearTest(parameterized.TestCase):
     )
     np.testing.assert_allclose(z, 3.0 * jnp.arange(4))
 
-  @parameterized.product(num_embedding=(1, 4))
-  def test_embed_nan(self, num_embedding):
-    rng = dict(params=random.key(0))
-    x = jnp.zeros(4)[None].astype(jnp.int32)
-    dummy_embedding = jnp.arange(3)[None, ...].astype(jnp.float32)
-    dummy_embedding = jnp.broadcast_to(dummy_embedding, (num_embedding, 3))
-    embed_module = nn.Embed(
-      num_embeddings=num_embedding,
-      features=3,
-      embedding_init=lambda rng, shape, dtype: dummy_embedding,
-    )
-    y, initial_params = embed_module.init_with_output(rng, x)
-    np.testing.assert_allclose(
-      y, jnp.broadcast_to(dummy_embedding[0][None], (4, 3))[None]
-    )
-    x = jnp.arange(2)[None] + num_embedding - 1
-    y, initial_params = embed_module.init_with_output(rng, x)
-    expected = jnp.broadcast_to(
-      jnp.array(
-        [
-          [
-            [
-              False,
-            ],
-            [
-              True,
-            ],
-          ]
-        ]
-      ),
-      (1, 2, 3),
-    )
-    np.testing.assert_allclose(jnp.isnan(y), expected)
-
   def test_embed_hash(self):
     self.assertEqual(hash(nn.Embed(2, 3)), hash(nn.Embed(2, 3)))
     self.assertNotEqual(hash(nn.Embed(3, 4)), hash(nn.Embed(2, 3)))
