@@ -17,6 +17,7 @@
 import contextlib
 import copy
 import dataclasses
+import enum
 import functools
 import gc
 import inspect
@@ -25,16 +26,12 @@ import sys
 from tempfile import TemporaryDirectory
 from typing import (
   Any,
-  Callable,
   Generic,
-  Mapping,
   NamedTuple,
-  Optional,
-  Sequence,
-  Tuple,
   TypeVar,
   get_type_hints,
 )
+from collections.abc import Callable, Mapping, Sequence
 from unittest.mock import patch
 
 import jax
@@ -252,6 +249,36 @@ class ModuleTest(absltest.TestCase):
       param_shape, {'a_(1, 2)': {'kernel': (3, 2), 'bias': (2,)}}
     )
 
+  def test_setup_frozen_dict_nonstring_keys(self):
+    a = {1: 2}
+
+    class Foo(nn.Module):
+      def setup(self):
+        self.a = FrozenDict(a)  # int as key.
+
+      @nn.compact
+      def __call__(self, x):
+        return self.a[x]
+
+    foo = Foo()
+    x = 1
+    params = foo.init(random.key(0), x)
+    assert foo.apply(params, x) == a[x]
+
+  def test_setup_dict_nonstring_keys_in_state(self):
+    class Foo(nn.Module):
+      a: dict[int, int]  # int as key.
+
+      @nn.compact
+      def __call__(self, x):
+        return self.a[x]
+
+    a = {1: 2}
+    foo = Foo(a)
+    x = 1
+    params = foo.init(random.key(0), x)
+    assert foo.apply(params, x) == a[x]
+
   def test_setup_cloning(self):
     class MLP(nn.Module):
       def setup(self):
@@ -295,7 +322,7 @@ class ModuleTest(absltest.TestCase):
     rngkey = jax.random.key(0)
 
     class DummyModuleWithoutCompact(nn.Module):
-      xshape: Tuple[int, ...]
+      xshape: tuple[int, ...]
 
       def setup(self):
         self.bias = self.param('bias', initializers.ones, self.xshape)
@@ -347,7 +374,7 @@ class ModuleTest(absltest.TestCase):
     rngkey = jax.random.key(0)
 
     class Dummy(nn.Module):
-      xshape: Tuple[int, ...]
+      xshape: tuple[int, ...]
 
       def setup(self):
         self.bias = self.param('bias', initializers.ones, self.xshape)
@@ -367,7 +394,7 @@ class ModuleTest(absltest.TestCase):
     rngkey = jax.random.key(0)
 
     class Dummy(nn.Module):
-      xshape: Tuple[int, ...]
+      xshape: tuple[int, ...]
 
       @compact
       def __call__(self, x):
@@ -385,7 +412,7 @@ class ModuleTest(absltest.TestCase):
     rngkey = jax.random.key(0)
 
     class Dummy(nn.Module):
-      xshape: Tuple[int, ...]
+      xshape: tuple[int, ...]
 
       def setup(self):
         self.bias = self.param('bias', initializers.ones, self.xshape)
@@ -404,7 +431,7 @@ class ModuleTest(absltest.TestCase):
     rngkey = jax.random.key(0)
 
     class Dummy(nn.Module):
-      xshape: Tuple[int, ...]
+      xshape: tuple[int, ...]
 
       def setup(self):
         self.biases = [
@@ -424,7 +451,7 @@ class ModuleTest(absltest.TestCase):
     rngkey = jax.random.key(0)
 
     class Dummy(nn.Module):
-      xshape: Tuple[int, ...]
+      xshape: tuple[int, ...]
 
       def setup(self):
         self.biases = {
@@ -449,7 +476,7 @@ class ModuleTest(absltest.TestCase):
     rngkey = jax.random.key(0)
 
     class Dummy(nn.Module):
-      xshape: Tuple[int, ...]
+      xshape: tuple[int, ...]
 
       def setup(self):
         self.bias = self.param('bias', initializers.ones, self.xshape)
@@ -468,7 +495,7 @@ class ModuleTest(absltest.TestCase):
     rngkey = jax.random.key(0)
 
     class Dummy(nn.Module):
-      xshape: Tuple[int, ...]
+      xshape: tuple[int, ...]
 
       def setup(self):
         self.bias = self.param('bias', initializers.ones, self.xshape)
@@ -489,7 +516,7 @@ class ModuleTest(absltest.TestCase):
     rngkey = jax.random.key(0)
 
     class Dummy(nn.Module):
-      xshape: Tuple[int, ...]
+      xshape: tuple[int, ...]
 
       def setup(self):
         self.bias = DummyModule()
@@ -516,18 +543,29 @@ class ModuleTest(absltest.TestCase):
 
     Foo({'a': ()}).apply({})
 
-  def test_only_one_compact_method(self):
-    msg = 'Only one method per class can be @compact'
-    with self.assertRaisesRegex(errors.MultipleMethodsCompactError, msg):
+  def test_multiple_compact_methods(self):
+    """Test that multiple methods with the @compact decorator can be used.
 
-      class MultipleCompactMethods(nn.Module):
-        @compact
-        def call1(self):
-          pass
+    NOTE: in the near future we might want to have compact methods reset the
+    autoname_cursor such that Dense would be reused in the second method.
+    """
 
-        @compact
-        def call2(self):
-          pass
+    class MultipleCompactMethods(nn.Module):
+      @compact
+      def __call__(self, x):
+        x = nn.Dense(1)(x)
+        return self.method(x)
+
+      @compact
+      def method(self, x):
+        x = nn.Dense(1)(x)
+        return x
+
+    m = MultipleCompactMethods()
+    variables = m.init(random.key(0), jnp.ones((1, 1)))
+    params = variables['params']
+    self.assertIn('Dense_0', params)
+    self.assertIn('Dense_1', params)
 
   def test_only_one_compact_method_subclass(self):
     class Dummy(nn.Module):
@@ -792,6 +830,7 @@ class ModuleTest(absltest.TestCase):
         precision = None
         kernel_init = init
         bias_init = zeros
+        promote_dtype = promote_dtype
         dot_general = None
         dot_general_cls = None
     )
@@ -804,6 +843,7 @@ class ModuleTest(absltest.TestCase):
         precision = None
         kernel_init = init
         bias_init = zeros
+        promote_dtype = promote_dtype
         dot_general = None
         dot_general_cls = None
     )
@@ -814,6 +854,61 @@ class ModuleTest(absltest.TestCase):
     trace = mlp.apply(variables, x)
     self.assertEqual(trace, expected_trace)
 
+  def test_default_params_rng_equivalence(self):
+    class Model(nn.Module):
+      @nn.compact
+      def __call__(self, x, add_dropout=False, add_noise=False):
+        x = nn.Dense(16)(x)
+        x = nn.Dropout(0.5)(x, deterministic=not add_dropout)
+        if add_noise:
+          x += jax.random.normal(self.make_rng('params'))
+        return x
+
+    model = Model()
+    key0, key1, key2 = jax.random.split(jax.random.key(0), 3)
+    x = jax.random.normal(key0, (10, 8))
+
+    with self.assertRaisesRegex(
+      ValueError,
+      'First argument passed to an init function should be a ``jax.PRNGKey``',
+    ):
+      model.init({'params': 'test'}, x)
+    with self.assertRaisesRegex(
+      errors.InvalidRngError,
+      'RNGs should be of shape \\(2,\\) or PRNGKey in Module Model, but rngs are: test',
+    ):
+      model.init('test', x)
+    # should not throw an error, since nn.Dropout will get an RNG key from the 'params' stream
+    model.init(key1, x, add_dropout=True)
+
+    v = model.init({'params': key1}, x)
+    v2 = model.init(key1, x)
+    jax.tree_util.tree_map(np.testing.assert_allclose, v, v2)
+
+    for add_dropout, add_noise in [[True, False], [False, True], [True, True]]:
+      out = model.apply(
+        v,
+        x,
+        add_dropout=add_dropout,
+        add_noise=add_noise,
+        rngs={'params': key2},
+      )
+      out2 = model.apply(
+        v, x, add_dropout=add_dropout, add_noise=add_noise, rngs=key2
+      )
+      np.testing.assert_allclose(out, out2)
+
+    with self.assertRaisesRegex(
+      ValueError,
+      'The ``rngs`` argument passed to an apply function should be a ``jax.PRNGKey``',
+    ):
+      model.apply(v, x, rngs={'params': 'test'})
+    with self.assertRaisesRegex(
+      errors.InvalidRngError,
+      'RNGs should be of shape \\(2,\\) or PRNGKey in Module Model, but rngs are: test',
+    ):
+      model.apply(v, x, rngs='test')
+
   def test_module_apply_method(self):
     class Foo(nn.Module):
       not_callable: int = 1
@@ -1449,6 +1544,34 @@ class ModuleTest(absltest.TestCase):
       all(intm_grads['after_multiply'] * 4 == intm_grads['before_multiply'])
     )
 
+  def test_perturb_setup(self):
+    class Foo(nn.Module):
+      def setup(self):
+        self.a = nn.Dense(10)
+
+      def __call__(self, x):
+        x = self.a(x)
+        x = self.perturb('before_multiply', x)
+        x = 4 * x
+        x = self.perturb('after_multiply', x)
+        return x
+
+    def loss(params, perturbations, inputs, targets):
+      variables = {'params': params, 'perturbations': perturbations}
+      preds = Foo().apply(variables, inputs)
+      return jnp.square(preds - targets).mean()
+
+    x = jax.random.uniform(jax.random.key(1), shape=(10,))
+    y = jax.random.uniform(jax.random.key(2), shape=(10,))
+    variables = Foo().init(jax.random.key(0), x)
+    intm_grads = jax.grad(loss, argnums=1)(
+      variables['params'], variables['perturbations'], x, y
+    )
+    # activation * 4 so reverse gradient also * 4
+    self.assertTrue(
+      all(intm_grads['after_multiply'] * 4 == intm_grads['before_multiply'])
+    )
+
   def test_perturb_noop(self):
     class Foo(nn.Module):
       @nn.compact
@@ -1469,9 +1592,7 @@ class ModuleTest(absltest.TestCase):
     module.apply({'params': params}, x)
 
     # check errors if perturbations is passed but empty
-    with self.assertRaisesRegex(
-      errors.ScopeCollectionNotFound, 'Tried to access'
-    ):
+    with self.assertRaisesRegex(ValueError, 'Perturbation collection'):
       module.apply({'params': params, 'perturbations': {}}, x)
 
     # check no error if perturbations is passed and not empty
@@ -1563,22 +1684,22 @@ class ModuleTest(absltest.TestCase):
     self.assertEqual(decoder.features, 2)
 
     self.assertTrue(
-      jax.tree_util.tree_all(
-        jax.tree_map(
-          lambda v1, v2: (v1 == v2).all(),
-          variables['params']['encoder'],
-          encoder_vars['params'],
+        jax.tree_util.tree_all(
+            jax.tree_util.tree_map(
+                lambda v1, v2: (v1 == v2).all(),
+                variables['params']['encoder'],
+                encoder_vars['params'],
+            )
         )
-      )
     )
     self.assertTrue(
-      jax.tree_util.tree_all(
-        jax.tree_map(
-          lambda v1, v2: (v1 == v2).all(),
-          variables['params']['decoder'],
-          decoder_vars['params'],
+        jax.tree_util.tree_all(
+            jax.tree_util.tree_map(
+                lambda v1, v2: (v1 == v2).all(),
+                variables['params']['decoder'],
+                decoder_vars['params'],
+            )
         )
-      )
     )
 
   def test_bind_unbind_equality(self):
@@ -1599,9 +1720,11 @@ class ModuleTest(absltest.TestCase):
     self.assertTrue((module.apply(variables, x) == bound_module(x)).all())
     new_module, new_variables = bound_module.unbind()
     self.assertTrue(
-      jax.tree_util.tree_all(
-        jax.tree_map(lambda v1, v2: (v1 == v2).all(), variables, new_variables)
-      )
+        jax.tree_util.tree_all(
+            jax.tree_util.tree_map(
+                lambda v1, v2: (v1 == v2).all(), variables, new_variables
+            )
+        )
     )
     self.assertEqual(module, new_module)
 
@@ -1692,7 +1815,7 @@ class ModuleTest(absltest.TestCase):
       submodule: MyComponent2[jnp.ndarray]
 
   def test_jit_rng_equivalance(self):
-    model = nn.Dense(1, use_bias=False)
+    model = nn.fold_rngs(nn.Dense)(1, use_bias=False)
     jit_model = nn.jit(nn.Dense)(1, use_bias=False)
     param = model.init(random.key(0), np.ones((1, 1)))['params']['kernel']
     param_2 = jit_model.init(random.key(0), np.ones((1, 1)))['params']['kernel']
@@ -2159,7 +2282,7 @@ class ModuleTest(absltest.TestCase):
   def test_kw_only(self):
     def create_kw_layers():
       class BaseLayer(nn.Module, kw_only=True):
-        base_multiplier: Optional[int] = -1
+        base_multiplier: int | None = -1
 
       class ChildLayer(BaseLayer):
         child_multiplier: int  # Don't want to have to set a default argument!
@@ -2462,6 +2585,17 @@ class ModuleTest(absltest.TestCase):
     self.assertIs(called[0], bar)
     self.assertIs(called[1], foo)
 
+  def test_cloudpickle_class(self):
+    import cloudpickle
+
+    class MyModule(nn.Module):
+      pass
+
+    a = MyModule()
+
+    UnpickledMyModule = cloudpickle.loads(cloudpickle.dumps(MyModule))
+    b = UnpickledMyModule()
+
   def test_cloudpickle_module(self):
     from cloudpickle import cloudpickle_fast
 
@@ -2554,47 +2688,45 @@ class ModuleTest(absltest.TestCase):
     model = Model()
 
     # test init equality
-    default_variables = model.init({'default': key1}, x, apply_dropout=False)
-    # adding 'default' rng shouldn't change anything
-    rngs = {'params': key1, 'var_rng': key1, 'noise': key1, 'default': key0}
+    default_variables = model.init({'params': key1}, x, apply_dropout=False)
+    rngs = {'params': key1, 'var_rng': key1, 'noise': key1}
     explicit_variables = model.init(rngs, x, apply_dropout=False)
     self.assertTrue(
-      jax.tree_util.tree_all(
-        jax.tree_map(
-          lambda v1, v2: (v1 == v2).all(), default_variables, explicit_variables
+        jax.tree_util.tree_all(
+            jax.tree_util.tree_map(
+                lambda v1, v2: (v1 == v2).all(),
+                default_variables,
+                explicit_variables,
+            )
         )
-      )
     )
 
     # test init inequality
-    rngs['default'] = key1  # adding 'default' rng shouldn't change anything
     for rng_name in ('params', 'var_rng'):
       rngs[rng_name] = key2
       explicit_variables = model.init(rngs, x, apply_dropout=False)
       self.assertFalse(
-        jax.tree_util.tree_all(
-          jax.tree_map(
-            lambda v1, v2: (v1 == v2).all(),
-            default_variables,
-            explicit_variables,
+          jax.tree_util.tree_all(
+              jax.tree_util.tree_map(
+                  lambda v1, v2: (v1 == v2).all(),
+                  default_variables,
+                  explicit_variables,
+              )
           )
-        )
       )
       rngs[rng_name] = key1
 
     # test apply equality
     default_out = model.apply(
-      default_variables, x, apply_dropout=True, rngs={'default': key1}
+      default_variables, x, apply_dropout=True, rngs={'params': key1}
     )
-    # adding 'default' rng shouldn't change anything
-    rngs = {'dropout': key1, 'noise': key1, 'default': key0}
+    rngs = {'dropout': key1, 'noise': key1}
     explicit_out = model.apply(
       default_variables, x, apply_dropout=True, rngs=rngs
     )
     np.testing.assert_allclose(default_out, explicit_out)
 
     # test apply inequality
-    rngs['default'] = key1  # adding 'default' rng shouldn't change anything
     for rng_name in ('dropout', 'noise'):
       rngs[rng_name] = key2
       explicit_out = model.apply(
@@ -2620,22 +2752,22 @@ class ModuleTest(absltest.TestCase):
 
     key0, key1 = jax.random.split(jax.random.key(0), 2)
     x = jax.random.normal(key0, (10, 4))
-    default_out = Model().apply({}, x, rngs={'default': key1})
+    default_out = Model().apply({}, x, rngs={'params': key1})
 
     class SubModel(nn.Module):
       @nn.compact
       def __call__(self, x):
-        noise = jax.random.normal(self.make_rng('default'), x.shape)
+        noise = jax.random.normal(self.make_rng('params'), x.shape)
         return x + noise
 
     class Model(nn.Module):
       @nn.compact
       def __call__(self, x):
         x = SubModel()(x)
-        noise = jax.random.normal(self.make_rng('default'), x.shape)
+        noise = jax.random.normal(self.make_rng('params'), x.shape)
         return x + noise
 
-    explicit_out = Model().apply({}, x, rngs={'default': key1})
+    explicit_out = Model().apply({}, x, rngs={'params': key1})
     np.testing.assert_allclose(default_out, explicit_out)
 
   def test_default_rng_error(self):
@@ -2653,13 +2785,13 @@ class ModuleTest(absltest.TestCase):
     class Model(nn.Module):
       @nn.compact
       def __call__(self, x):
-        return x + jax.random.normal(self.make_rng('default'), x.shape)
+        return x + jax.random.normal(self.make_rng(), x.shape)
 
     model = Model()
     with self.assertRaisesRegex(
-      errors.InvalidRngError, 'None needs PRNG for "default"'
+      errors.InvalidRngError, 'None needs PRNG for "params"'
     ):
-      model.init(jax.random.key(0), jnp.ones((1, 3)))
+      model.init({'other_rng_stream': jax.random.key(0)}, jnp.ones((1, 3)))
 
   def test_compact_name_scope(self):
     class Foo(nn.Module):
@@ -3037,6 +3169,20 @@ class RelaxedNamingTests(absltest.TestCase):
     self.assertIn('next_layer_1', variables['params'])
     self.assertNotIn('child_template', variables['params'])
 
+  def test_nonstring_keys_in_dict_on_module(self):
+    class MyEnum(str, enum.Enum):
+      a = 'a'
+      b = 'b'
+
+    class MyModule(nn.Module):
+      config: dict[MyEnum, int]
+
+      def __call__(self, inputs):
+        return inputs
+
+    module = MyModule(config={MyEnum.a: 1, MyEnum.b: 2})
+    variables = module.init(jax.random.key(0), jnp.zeros([0]))
+
 
 class FrozenDictTests(absltest.TestCase):
   def test_frozendict_flag(self):
@@ -3053,5 +3199,174 @@ class FrozenDictTests(absltest.TestCase):
       self.assertTrue(isinstance(params, dict))
 
 
+class ShareScopeTest(absltest.TestCase):
+  def test_basic(self):
+    class DenseLoRA(nn.Module):
+      inner: nn.Dense
+      rank: int
+
+      def setup(self):
+        nn.share_scope(self, self.inner)
+
+      @nn.compact
+      def __call__(self, x: jax.Array):
+        din, dout = x.shape[-1], self.inner.features
+        A = self.param('A', nn.zeros_init(), (din, self.rank))
+        B = self.param('B', nn.zeros_init(), (self.rank, dout))
+        return self.inner(x) + x @ A @ B
+
+    dense_lora = DenseLoRA(nn.Dense(10), rank=2)
+
+    params = dense_lora.init(random.key(0), jnp.ones((1, 5)))['params']
+
+    self.assertIn('kernel', params)
+    self.assertIn('bias', params)
+    self.assertIn('A', params)
+    self.assertIn('B', params)
+
+  def test_child_scope(self):
+    class DenseLoRA(nn.Module):
+      rank: int
+
+      def setup(self):
+        self.child = nn.Dense(10)
+        nn.share_scope(self, self.child)
+
+      @nn.compact
+      def __call__(self, x: jax.Array):
+        din, dout = x.shape[-1], self.child.features
+        A = self.param('A', nn.zeros_init(), (din, self.rank))
+        B = self.param('B', nn.zeros_init(), (self.rank, dout))
+        return self.child(x) + x @ A @ B
+
+    dense_lora = DenseLoRA(rank=2)
+
+    params = dense_lora.init(random.key(0), jnp.ones((1, 5)))['params']
+
+    self.assertIn('kernel', params)
+    self.assertIn('bias', params)
+    self.assertIn('A', params)
+    self.assertIn('B', params)
+
+  def test_in_compact(self):
+    class DenseLoRA(nn.Module):
+      rank: int
+
+      def setup(self):
+        self.child = nn.Dense(10)
+        nn.share_scope(self, self.child)
+
+      @nn.compact
+      def __call__(self, x: jax.Array):
+        din, dout = x.shape[-1], self.child.features
+        A = self.param('A', nn.zeros_init(), (din, self.rank))
+        B = self.param('B', nn.zeros_init(), (self.rank, dout))
+        return self.child(x) + x @ A @ B
+
+    class Model(nn.Module):
+      @nn.compact
+      def __call__(self, x: jax.Array):
+        return DenseLoRA(rank=2)(x)
+
+    model = Model()
+
+    params = model.init(random.key(0), jnp.ones((1, 5)))['params']
+
+    self.assertIn('kernel', params['DenseLoRA_0'])
+    self.assertIn('bias', params['DenseLoRA_0'])
+    self.assertIn('A', params['DenseLoRA_0'])
+    self.assertIn('B', params['DenseLoRA_0'])
+
+  def test_adopt_child_name(self):
+    class DenseLoRA(nn.Module):
+      inner: nn.Dense
+      rank: int
+
+      def setup(self):
+        nn.share_scope(self, self.inner)
+
+      @nn.compact
+      def __call__(self, x: jax.Array):
+        din, dout = x.shape[-1], self.inner.features
+        A = self.param('A', nn.zeros_init(), (din, self.rank))
+        B = self.param('B', nn.zeros_init(), (self.rank, dout))
+        return self.inner(x) + x @ A @ B
+
+    class Model(nn.Module):
+      @nn.compact
+      def __call__(self, x: jax.Array):
+        return DenseLoRA(nn.Dense(10), rank=2)(x)
+
+    model = Model()
+
+    params = model.init(random.key(0), jnp.ones((1, 5)))['params']
+
+    self.assertIn('kernel', params['Dense_0'])
+    self.assertIn('bias', params['Dense_0'])
+    self.assertIn('A', params['Dense_0'])
+    self.assertIn('B', params['Dense_0'])
+
+  def test_other_scope_is_none(self):
+    class DenseLoRA(nn.Module):
+      inner: nn.Dense
+      rank: int
+
+      def setup(self):
+        nn.share_scope(self, self.inner)
+
+      @nn.compact
+      def __call__(self, x: jax.Array):
+        din, dout = x.shape[-1], self.inner.features
+        A = self.param('A', nn.zeros_init(), (din, self.rank))
+        B = self.param('B', nn.zeros_init(), (self.rank, dout))
+        return self.inner(x) + x @ A @ B
+
+    class Model(nn.Module):
+      def setup(self):
+        # here Dense doesn't have a scope yet
+        self.dense_lora = DenseLoRA(nn.Dense(10), rank=2)
+
+      @nn.compact
+      def __call__(self, x: jax.Array):
+        return self.dense_lora(x)
+
+    model = Model()
+
+    params = model.init(random.key(0), jnp.ones((1, 5)))['params']
+
+    self.assertIn('kernel', params['dense_lora'])
+    self.assertIn('bias', params['dense_lora'])
+    self.assertIn('A', params['dense_lora'])
+    self.assertIn('B', params['dense_lora'])
+
+  def test_external_grandchild_scope_correct(self):
+    class GrandChild(nn.Module):
+      @nn.compact
+      def __call__(self):
+        return nn.Dense(50)(jnp.zeros(10))
+
+    class Child(nn.Module):
+      child: GrandChild
+
+      @nn.compact
+      def __call__(self, *args: Any, **kwargs: Any) -> Any:
+        return self.child(*args, **kwargs)
+
+    class Parent(nn.Module):
+      main_child: Child
+
+      def setup(self):
+        nn.share_scope(self, self.main_child)
+
+      @nn.compact
+      def __call__(self, *args: Any, **kwargs: Any) -> Any:
+        nn.Dense(10)(jnp.zeros(10))
+        r = self.main_child(*args, **kwargs)
+        return r
+
+    params = Parent(Child(GrandChild())).init(jax.random.key(0))
+    self.assertNotIn('main_child', params['params'])
+    self.assertIn('child', params['params'])
+
 if __name__ == '__main__':
   absltest.main()
