@@ -146,7 +146,7 @@ class NormalizationTest(parameterized.TestCase):
     x = jnp.where(m, x, jnp.nan)
 
     module = nn.LayerNorm()
-    y, w = module.init_with_output(key, x, m)
+    y, w = module.init_with_output(key, x, mask=m)
 
     z = y.mean(-1, where=m)
     np.testing.assert_allclose(z, 0, atol=1e-4)
@@ -163,7 +163,7 @@ class NormalizationTest(parameterized.TestCase):
     x = jnp.where(m, x, jnp.nan)
 
     module = nn.RMSNorm()
-    y, w = module.init_with_output(key, x, m)
+    y, w = module.init_with_output(key, x, mask=m)
 
     z = np.square(y).mean(-1, where=m)
     np.testing.assert_allclose(z, 1, atol=1e-4)
@@ -177,7 +177,7 @@ class NormalizationTest(parameterized.TestCase):
     x = jnp.where(m, x, jnp.nan)
 
     module = nn.GroupNorm(7, use_bias=False, use_scale=False)
-    y, w = module.init_with_output(key, x, m)
+    y, w = module.init_with_output(key, x, mask=m)
 
     yr = y.reshape((13, 3, 5, 7, 11))
     mr = m.reshape((13, 3, 5, 7, 11))
@@ -287,7 +287,7 @@ class NormalizationTest(parameterized.TestCase):
     y_one_liner = (
       x - x.mean(axis=reduction_axes, keepdims=True)
     ) * jax.lax.rsqrt(x.var(axis=reduction_axes, keepdims=True) + e)
-    np.testing.assert_allclose(y_one_liner, y, atol=1e-4)
+    np.testing.assert_allclose(y_one_liner, y, atol=1e-3, rtol=1e-3)
 
   @parameterized.parameters(
     {'reduction_axes': -1}, {'reduction_axes': 1}, {'reduction_axes': (1, 2)}
@@ -428,7 +428,7 @@ class NormalizationTest(parameterized.TestCase):
     model = Foo()
     x = random.normal(random.key(1), (2, 4))
     (y1, y2), _ = model.init_with_output(key, x)
-    np.testing.assert_allclose(y1, y2, rtol=1e-4)
+    np.testing.assert_allclose(y1, y2, rtol=0.005)
 
   @parameterized.parameters(
     {'feature_axes': -1},
@@ -537,11 +537,13 @@ class NormalizationTest(parameterized.TestCase):
     layer1_variables = layer1.init(jax.random.key(1), x)
     layer2_variables = layer2.init(jax.random.key(1), x)
     self.assertTrue(
-      jax.tree_util.tree_all(
-        jax.tree_map(
-          lambda v1, v2: (v1 == v2).all(), layer1_variables, layer2_variables
+        jax.tree_util.tree_all(
+            jax.tree_util.tree_map(
+                lambda v1, v2: (v1 == v2).all(),
+                layer1_variables,
+                layer2_variables,
+            )
         )
-      )
     )
 
     layer1_y = layer1.apply(layer1_variables, x)
@@ -673,7 +675,7 @@ class NormalizationTest(parameterized.TestCase):
     model_cls = Foo()
     variables = model_cls.init(random.PRNGKey(0), x, train=False)
     params, batch_stats = variables['params'], variables['batch_stats']
-    params = jax.tree_map(lambda x: 4 * jnp.eye(*x.shape), params)
+    params = jax.tree_util.tree_map(lambda x: 4 * jnp.eye(*x.shape), params)
     _, updates = model_cls.apply(
       {'params': params, 'batch_stats': batch_stats},
       x=x,
@@ -747,7 +749,7 @@ class NormalizationTest(parameterized.TestCase):
     x = jax.random.normal(key1, (1, 3))
     module = Foo()
     v = module.init(key2, x)
-    v = jax.tree_map(lambda x: x + 0.5, v)
+    v = jax.tree_util.tree_map(lambda x: x + 0.5, v)
     out = module.apply(v, x)
 
     kernel = v['params']['Dense_0']['kernel']
@@ -918,7 +920,7 @@ class NormalizationTest(parameterized.TestCase):
       prev_loss = loss
 
 
-class StochasticTest(absltest.TestCase):
+class StochasticTest(parameterized.TestCase):
   def test_dropout(self):
     rng = random.key(0)
     key1, key2 = random.split(rng)
@@ -973,13 +975,77 @@ class StochasticTest(absltest.TestCase):
     res = jax.grad(lambda x, k: jnp.sum(fn(x, k)))(inputs, key3)
     self.assertFalse(np.isnan(res).any())
 
+  @parameterized.parameters(
+    {
+      'num_dims': 2,
+      'broadcast_dims': (1,),
+      'slice_fn': lambda out, i: out[i, :],
+      'summed_total': 2 * 10,
+    },
+    {
+      'num_dims': 2,
+      'broadcast_dims': (0,),
+      'slice_fn': lambda out, i: out[:, i],
+      'summed_total': 2 * 10,
+    },
+    {
+      'num_dims': 3,
+      'broadcast_dims': (1, 2),
+      'slice_fn': lambda out, i: out[i, :, :],
+      'summed_total': 2 * 10 * 10,
+    },
+    {
+      'num_dims': 3,
+      'broadcast_dims': (1,),
+      'slice_fn': lambda out, i, j: out[i, :, j],
+      'summed_total': 2 * 10,
+    },
+    {
+      'num_dims': 4,
+      'broadcast_dims': (0, 2, 3),
+      'slice_fn': lambda out, i: out[:, i, :, :],
+      'summed_total': 2 * 10 * 10 * 10,
+    },
+    {
+      'num_dims': 4,
+      'broadcast_dims': (0, 1),
+      'slice_fn': lambda out, i, j: out[:, :, i, j],
+      'summed_total': 2 * 10 * 10,
+    },
+    {
+      'num_dims': 4,
+      'broadcast_dims': (3,),
+      'slice_fn': lambda out, i, j, k: out[i, j, k, :],
+      'summed_total': 2 * 10,
+    },
+  )
+  def test_dropout_broadcast(
+    self, num_dims, broadcast_dims, slice_fn, summed_total
+  ):
+    module = nn.Dropout(
+      rate=0.5, broadcast_dims=broadcast_dims, deterministic=False
+    )
+    x = jnp.ones((10,) * num_dims)
+    out = module.apply({}, x, rngs={'dropout': random.key(0)})
+
+    for i in range(10):
+      if num_dims - len(broadcast_dims) >= 2:
+        for j in range(10):
+          if num_dims - len(broadcast_dims) >= 3:
+            for k in range(10):
+              self.assertTrue(slice_fn(out, i, j, k).sum() in (0, summed_total))
+          else:
+            self.assertTrue(slice_fn(out, i, j).sum() in (0, summed_total))
+      else:
+        self.assertTrue(slice_fn(out, i).sum() in (0, summed_total))
+
   def test_dropout_manual_rng(self):
     class Foo(nn.Module):
       @nn.compact
       def __call__(self, x):
         key = self.make_rng('dropout')
         x1 = nn.Dropout(rate=0.5, deterministic=False)(x, rng=key)
-        x2 = nn.Dropout(rate=0.5, deterministic=False)(x, rng=key)
+        x2 = nn.Dropout(rate=0.5, deterministic=False)(x, rng=jax.random.clone(key))
         return x1, x2
 
     module = Foo()
@@ -995,7 +1061,7 @@ class RecurrentTest(parameterized.TestCase):
   def test_lstm(self):
     lstm = nn.LSTMCell(features=4)
     rng = random.key(0)
-    key1, key2 = random.split(rng)
+    rng, key1, key2 = random.split(rng, 3)
     x = random.normal(key1, (2, 3))
     c0, h0 = lstm.initialize_carry(rng, x.shape)
     self.assertEqual(c0.shape, (2, 4))
@@ -1020,6 +1086,13 @@ class RecurrentTest(parameterized.TestCase):
     )
 
   @parameterized.parameters(
+    {
+      'module_cls': nn.SimpleCell,
+      'expected_param_shapes': {
+        'i': {'kernel': (3, 4), 'bias': (4,)},
+        'h': {'kernel': (4, 4)},
+      },
+    },
     {
       'module_cls': nn.GRUCell,
       'expected_param_shapes': {
@@ -1044,7 +1117,7 @@ class RecurrentTest(parameterized.TestCase):
   def test_gated_units(self, module_cls, expected_param_shapes):
     module = module_cls(features=4)
     rng = random.key(0)
-    key1, key2 = random.split(rng)
+    rng, key1, key2 = random.split(rng, 3)
     x = random.normal(key1, (2, 3))
     carry0 = module.initialize_carry(rng, x.shape)
     self.assertEqual(carry0.shape, (2, 4))
@@ -1068,12 +1141,14 @@ class RecurrentTest(parameterized.TestCase):
       )
 
   @parameterized.parameters(
-    {'module_cls': nn.GRUCell}, {'module_cls': nn.MGUCell}
+    {'module_cls': nn.SimpleCell},
+    {'module_cls': nn.GRUCell},
+    {'module_cls': nn.MGUCell},
   )
   def test_complex_input_gated_units(self, module_cls):
     module_instance = module_cls(features=4)
     rng = random.key(0)
-    key1, key2 = random.split(rng)
+    rng, key1, key2 = random.split(rng, 3)
     x = random.normal(key1, (2, 3), dtype=jnp.complex64)
     carry0 = module_instance.initialize_carry(rng, x.shape)
     self.assertEqual(carry0.shape, (2, 4))
@@ -1084,7 +1159,7 @@ class RecurrentTest(parameterized.TestCase):
   def test_convlstm(self):
     lstm = nn.ConvLSTMCell(features=6, kernel_size=(3, 3))
     rng = random.key(0)
-    key1, key2 = random.split(rng)
+    rng, key1, key2 = random.split(rng, 3)
     x = random.normal(key1, (2, 4, 4, 3))
     c0, h0 = lstm.initialize_carry(rng, x.shape)
     self.assertEqual(c0.shape, (2, 4, 4, 6))
@@ -1106,7 +1181,7 @@ class RecurrentTest(parameterized.TestCase):
     # Create regular LSTMCell.
     lstm = nn.LSTMCell(features=4)
     rng = random.key(0)
-    key1, key2 = random.split(rng)
+    rng, key1, key2 = random.split(rng, 3)
     x = random.normal(key1, (2, 3))
     c0, h0 = lstm.initialize_carry(rng, x.shape)
     self.assertEqual(c0.shape, (2, 4))
@@ -1116,7 +1191,7 @@ class RecurrentTest(parameterized.TestCase):
     # Create OptimizedLSTMCell.
     lstm_opt = nn.OptimizedLSTMCell(features=4)
     rng = random.key(0)
-    key1, key2 = random.split(rng)
+    rng, key1, key2 = random.split(rng, 3)
     x = random.normal(key1, (2, 3))
     c0, h0 = lstm_opt.initialize_carry(rng, x.shape)
     self.assertEqual(c0.shape, (2, 4))
@@ -1126,6 +1201,30 @@ class RecurrentTest(parameterized.TestCase):
     np.testing.assert_allclose(y, y_opt, rtol=1e-6)
     check_eq(lstm_params, lstm_opt_params)
 
+  def test_mgu_reset_gate(self):
+    module = nn.MGUCell(features=4, reset_gate=False)
+    rng = random.key(0)
+    rng, key1, key2 = random.split(rng, 3)
+    x = random.normal(key1, (2, 3))
+    carry0 = module.initialize_carry(rng, x.shape)
+    (carry, y), v = module.init_with_output(key2, carry0, x)
+
+    self.assertIn('kernel', v['params']['hn'])
+    self.assertNotIn('bias', v['params']['hn'])
+
+    f = jax.nn.sigmoid(
+      jnp.dot(x, v['params']['if']['kernel'])
+      + v['params']['if']['bias'].reshape(1, -1)
+      + jnp.dot(carry0, v['params']['hf']['kernel'])
+    )
+    n = jax.nn.tanh(
+      jnp.dot(x, v['params']['in']['kernel'])
+      + v['params']['in']['bias'].reshape(1, -1)
+      + jnp.dot(carry0, v['params']['hn']['kernel'])
+    )
+    expected_out = (1 - f) * n + f * carry0
+    np.testing.assert_allclose(y, expected_out)
+
 
 class IdsTest(absltest.TestCase):
   def test_hashable(self):
@@ -1139,30 +1238,90 @@ class IdsTest(absltest.TestCase):
     self.assertNotEqual(hash(id1), hash(id1c))
     self.assertNotEqual(hash(id1), hash(id1dc))
 
+def get_fp8_dtypes(fp8_genre):
+    assert fp8_genre in ('OCP', 'NANOO')
+    if fp8_genre == 'OCP':
+      e4m3_dtype = jnp.float8_e4m3fn
+      e5m2_dtype = jnp.float8_e5m2
+    else: # fp8_genre == 'NANOO'
+      e4m3_dtype = jnp.float8_e4m3fnuz
+      e5m2_dtype = jnp.float8_e5m2fnuz
+    return e4m3_dtype, e5m2_dtype
 
-class Fp8Test(absltest.TestCase):
-  def test_fp8_dot_general_injection(self):
+class Fp8Test(parameterized.TestCase):
+  @parameterized.parameters(
+    {'x_shape': (16, 32), 'y_shape': (32, 64), 'g_shape': (16, 64), 'eqn': 'mk,kn->mn'},
+    {'x_shape': (2, 3, 32), 'y_shape': (64, 32), 'g_shape': (2, 3, 64), 'eqn': '...k,nk->...n'},
+    {'x_shape': (2, 3, 64), 'y_shape': (64, 32), 'g_shape': (2, 3, 32), 'eqn': '...k,kn->...n'},
+  )
+  def test_fp8_einsum(self, x_shape, y_shape, g_shape, eqn):
+    rng, key1, key2, key3 = random.split(random.key(42), 4)
+    x = random.normal(key1, x_shape)
+    y = random.normal(key2, y_shape)
+    g = random.normal(key3, g_shape)
+    e4m3_dtype = jnp.float8_e4m3fn
+    e5m2_dtype = jnp.float8_e5m2
+    cast_to_representable = functools.partial(
+        fp8_ops.qdq,
+        scale=jnp.ones((1,)),
+        compute_dtype=jnp.float32,
+    )
+
+    x = cast_to_representable(x, e4m3_dtype)
+    y = cast_to_representable(y, e4m3_dtype)
+    g = cast_to_representable(g, e5m2_dtype)
+
+    p = nn.Fp8Einsum()
+    vars = p.init(rng, eqn, x, y)
+    def loss_fn(vars, x, y):
+      out = p.apply(vars, eqn, x, y)
+      return jnp.sum(out * g.astype(out.dtype))
+    step_fn = jax.value_and_grad(loss_fn, argnums=[1, 2])
+    out, grads = jax.jit(step_fn)(vars, x, y)
+
+    def loss_fn_ref(x, y):
+      out = jnp.einsum(eqn, x, y)
+      return jnp.sum(out * g.astype(out.dtype))
+    step_fn_ref = jax.value_and_grad(loss_fn_ref, argnums=[0, 1])
+    out_ref, grads_ref = jax.jit(step_fn_ref)(x, y)
+
+    np.testing.assert_allclose(out, out_ref, atol=1e-02, rtol=1e-02)
+    np.testing.assert_allclose(grads[0], grads_ref[0], atol=1e-02, rtol=1e-02)
+    np.testing.assert_allclose(grads[1], grads_ref[1], atol=1e-02, rtol=1e-02)
+
+
+  @parameterized.parameters(
+    {'fp8_genre': 'OCP'}, {'fp8_genre': 'NANOO'}
+  )
+  def test_fp8_dot_general_injection(self, fp8_genre):
     # Used to cast the inputs to be representable in FP8, so that the difference
     # of the results from the original gemm and fp8 gemm is small.
     cast_to_representable = functools.partial(
-      fp8_ops.quantize_dequantize,
+      fp8_ops.qdq,
       scale=jnp.ones((1,)),
       compute_dtype=jnp.float32,
     )
 
+    e4m3_dtype, e5m2_dtype = get_fp8_dtypes(fp8_genre)
+
     init_key, random_key = random.split(random.PRNGKey(seed=123), 2)
     x = cast_to_representable(
-      random.uniform(random_key, (16, 32)), jnp.float8_e4m3fn
+      random.uniform(random_key, (16, 32)), e4m3_dtype
     )
     dy = cast_to_representable(
-      random.uniform(random_key, (16, 64)), jnp.float8_e5m2
+      random.uniform(random_key, (16, 64)), e5m2_dtype
     )
 
+    quant_cls = nn.Fp8DotGeneral if fp8_genre == 'OCP' else nn.NANOOFp8DotGeneralOp
+
     def run(fp8_injection, expected_shapes):
       p = nn.DenseGeneral(features=64, name='dense')
+
       if fp8_injection:
-        p.dot_general_cls = nn.Fp8DotGeneralOp
-      y, initial_vars = p.init_with_output(init_key, x)
+        p.dot_general_cls = quant_cls
+
+      init_fn = jax.jit(p.init_with_output)
+      y, initial_vars = init_fn(init_key, x)
       var_shapes = jax.tree_util.tree_map(jnp.shape, initial_vars)
       self.assertEqual(var_shapes, expected_shapes)
 
@@ -1178,10 +1337,11 @@ class Fp8Test(absltest.TestCase):
     expected_shapes_original = {
       'params': {'kernel': (32, 64), 'bias': (64,)},
     }
+
     expected_shapes_new = {
       'params': {'kernel': (32, 64), 'bias': (64,)},
       fp8_ops.OVERWRITE_WITH_GRADIENT: {
-        'Fp8DotGeneralOp_0': {
+        f'{quant_cls.__name__}_0': {
           'input_amax_history': (1024,),
           'kernel_amax_history': (1024,),
           'output_grad_amax_history': (1024,),
@@ -1191,7 +1351,6 @@ class Fp8Test(absltest.TestCase):
         }
       },
     }
-
     output1a, output1b = run(False, expected_shapes_original)
     output2a, output2b = run(True, expected_shapes_new)
     dw1, dw2 = output1b[0]['params']['kernel'], output2b[0]['params']['kernel']
@@ -1201,13 +1360,20 @@ class Fp8Test(absltest.TestCase):
     np.testing.assert_allclose(dw1, dw2, atol=1e-04)
     np.testing.assert_allclose(dx1, dx2, atol=1e-04)
 
-  def test_fp8_train_state(self):
+  @parameterized.parameters(
+    {'fp8_genre': 'OCP'}, {'fp8_genre': 'NANOO'}
+  )
+  def test_fp8_train_state(self, fp8_genre):
     key, init_key, random_key = random.split(random.PRNGKey(seed=123), 3)
     x = random.uniform(random_key, (16, 16), dtype=jnp.float32)
+
+    quant_cls = nn.Fp8DotGeneral if fp8_genre == 'OCP' else nn.NANOOFp8DotGeneralOp
     dense = nn.DenseGeneral(
-      features=32, use_bias=True, dot_general_cls=nn.Fp8DotGeneralOp
+      features=32, use_bias=True, dot_general_cls=quant_cls
     )
-    variables = dense.init(init_key, x)
+
+    init_fn = jax.jit(dense.init)
+    variables = init_fn(init_key, x)
     opt = optax.adam(learning_rate=0.1)
     state = train_state.TrainState.create(
       params=variables, tx=opt, apply_fn=dense.apply
@@ -1232,8 +1398,9 @@ class Fp8Test(absltest.TestCase):
     scale_x, amax_history_x = jnp.ones(()), jnp.zeros((1024,))
     scale_k, amax_history_k = jnp.ones(()), jnp.zeros((1024,))
     scale_g, amax_history_g = jnp.ones(()), jnp.zeros((1024,))
-    e4m3_max = jnp.finfo(jnp.float8_e4m3fn).max.astype(jnp.float32)
-    e5m2_max = jnp.finfo(jnp.float8_e5m2).max.astype(jnp.float32)
+    e4m3_dtype, e5m2_dtype = get_fp8_dtypes(fp8_genre)
+    e4m3_max = jnp.finfo(e4m3_dtype).max.astype(jnp.float32)
+    e5m2_max = jnp.finfo(e5m2_dtype).max.astype(jnp.float32)
 
     for _ in range(5):
       key, random_key = random.split(key, 2)
@@ -1242,21 +1409,21 @@ class Fp8Test(absltest.TestCase):
       k = state.params['params']['kernel']
 
       # Manually compute the expected amax history and scaling factors.
-      amax_history_x = _roll_and_update(amax_history_x, jnp.max(jnp.abs(x)))
-      amax_history_k = _roll_and_update(amax_history_k, jnp.max(jnp.abs(k)))
-      amax_history_g = _roll_and_update(amax_history_g, jnp.max(jnp.abs(g)))
       amax_from_history_x = jnp.max(amax_history_x, axis=0)
       amax_from_history_k = jnp.max(amax_history_k, axis=0)
       amax_from_history_g = jnp.max(amax_history_g, axis=0)
       scale_x = fp8_ops.compute_scale(amax_from_history_x, scale_x, e4m3_max)
       scale_k = fp8_ops.compute_scale(amax_from_history_k, scale_k, e4m3_max)
       scale_g = fp8_ops.compute_scale(amax_from_history_g, scale_g, e5m2_max)
+      amax_history_x = _roll_and_update(amax_history_x, jnp.max(jnp.abs(x)))
+      amax_history_k = _roll_and_update(amax_history_k, jnp.max(jnp.abs(k)))
+      amax_history_g = _roll_and_update(amax_history_g, jnp.max(jnp.abs(g)))
 
       state = train_fn(state, x, g)
 
       rtol, atol = 0.001, 0.001
       fp8_vars = state.params[fp8_ops.OVERWRITE_WITH_GRADIENT][
-        'Fp8DotGeneralOp_0'
+        f'{quant_cls.__name__}_0'
       ]
       np.testing.assert_allclose(
         fp8_vars['input_amax_history'],
@@ -1281,6 +1448,50 @@ class Fp8Test(absltest.TestCase):
       np.testing.assert_allclose(fp8_vars['kernel_scale'][0], scale_k)
       np.testing.assert_allclose(fp8_vars['output_grad_scale'][0], scale_g)
 
+  @parameterized.parameters(
+          {'fp8_genre': 'OCP', 'use_jit': True},
+          {'fp8_genre': 'OCP', 'use_jit': False},
+          {'fp8_genre': 'NANOO', 'use_jit': True},
+          {'fp8_genre': 'NANOO', 'use_jit': False}
+  )
+  def test_fp8_meta_dtype(self, fp8_genre, use_jit):
+    if not use_jit and not fp8_ops.CAN_USE_EARRAY:
+      self.skipTest("TODO: requires newer jax that has earray")
+    f32 = jnp.dtype('float32')
+    fmax32 = fp8_ops.fp32_max_grad
+    e4m3_dtype, _ = get_fp8_dtypes(fp8_genre)
+    e4m3_max = 448 if fp8_genre == 'OCP' else 240
+
+    # Create a scan loop with reused ah_f32 and sf_f32. So, the autograd will
+    # accumulate the grads of them. We expect the max op (rather than add op)
+    # for the accumulation by converting them to fmax32 dtype.
+    def outer(x, ah_f32, sf_f32):
+      ah_fmax32 = jax.lax.convert_element_type(ah_f32, fmax32)
+      sf_fmax32 = jax.lax.convert_element_type(sf_f32, fmax32)
+      array_x = jnp.array([x], f32)
+      def body_fun(carry, _):
+        carry = fp8_ops.in_qdq(f32, e4m3_dtype, carry, sf_fmax32, ah_fmax32)
+        return carry, None
+      array_x, _ = jax.lax.scan(body_fun, array_x, None, length=3)
+      return array_x[0]
+
+    outer_fn = jax.grad(outer, (0, 1, 2))
+    if use_jit:
+      outer_fn = jax.jit(outer_fn)
+    ah = jnp.array([0., 0., 0.], f32)
+    sf = jnp.array([1.], f32)
+    # 1st iteration
+    grads, new_ah, new_sf = outer_fn(2.0, ah, sf)
+    np.testing.assert_allclose(new_ah, [2., 0., 0.])
+    np.testing.assert_allclose(new_sf, [1.])
+    # 2nd iteration
+    grads, new_ah, new_sf = outer_fn(3., new_ah, new_sf)
+    np.testing.assert_allclose(new_ah, [3., 0., 2.])
+    np.testing.assert_allclose(new_sf, [2. / e4m3_max])
+    # 3rd iteration
+    grads, new_ah, new_sf = outer_fn(4., new_ah, new_sf)
+    np.testing.assert_allclose(new_ah, [4., 2., 3.])
+    np.testing.assert_allclose(new_sf, [3. / e4m3_max])
 
 if __name__ == '__main__':
   absltest.main()
