@@ -16,14 +16,16 @@
 
 from functools import partial
 import operator
-from typing import Any, Callable, Sequence
+from typing import Any
+from collections.abc import Callable, Sequence
 import unittest
 
 from absl.testing import absltest, parameterized
 from flax import errors
 from flax import linen as nn
 from flax import serialization
-from flax.core import copy, freeze
+from flax import struct
+from flax.core import copy, freeze, AxisMetadata
 from flax.linen.transforms import _HashableProxy
 import jax
 from jax import random
@@ -84,6 +86,10 @@ def decorated_MLP(transform: Callable = id_fn):
 
 class TransformTest(parameterized.TestCase):
 
+  def assert_keys_equal(self, key1, key2):
+    self.assertEqual(key1.dtype, key2.dtype)
+    np.testing.assert_array_equal(random.key_data(key1), random.key_data(key2))
+
   def test_jit(self):
     key1, key2 = random.split(random.key(3), 2)
     x = random.uniform(key1, (4, 4))
@@ -108,6 +114,19 @@ class TransformTest(parameterized.TestCase):
 
     self.assertTrue(np.all(y1 == y2))
 
+  def test_jit_init_fn(self):
+    class Foo(nn.Module):
+      @nn.compact
+      def __call__(self, x):
+        return nn.Dense(2)(x)
+
+      @nn.jit
+      def init_with_output(self, rngs, *args, **kwargs):
+        return super().init_with_output(rngs, *args, **kwargs)
+
+    Foo().init_with_output(random.key(0), jnp.ones((2, 3)))
+
+
   def test_remat(self):
     key1, key2 = random.split(random.key(3), 2)
     x = random.uniform(key1, (4, 4))
@@ -221,9 +240,9 @@ class TransformTest(parameterized.TestCase):
     self.assertEqual(y.shape, (1, 3))
 
   def test_vmap(self):
-    key1, key2 = random.split(random.key(3), 2)
+    key1, key2, key3 = random.split(random.key(3), 3)
     x = random.uniform(key1, (4, 4))
-    x2 = random.uniform(key1, (5, 4, 4))
+    x2 = random.uniform(key2, (5, 4, 4))
 
     def vmap(cls):
       return nn.vmap(
@@ -235,19 +254,19 @@ class TransformTest(parameterized.TestCase):
 
     normal_model = TransformedMLP(features=[3, 4, 5])
     vmap_model = TransformedMLP(features=[3, 4, 5], transform=vmap)
-    init_variables = normal_model.init(key2, x)
+    init_variables = normal_model.init(key3, x)
     # simulate vmap in python for comparison:
     y1 = jnp.vstack([
         normal_model.apply(init_variables, x2[i])[None, ...]
         for i in np.arange(x2.shape[0])
     ])
     y2 = vmap_model.apply(init_variables, x2)
-    np.testing.assert_allclose(y1, y2, atol=1e-7)
+    np.testing.assert_allclose(y1, y2, atol=1e-6)
 
   def test_vmap_decorated(self):
-    key1, key2 = random.split(random.key(3), 2)
+    key1, key2, key3 = random.split(random.key(3), 3)
     x = random.uniform(key1, (4, 4))
-    x2 = random.uniform(key1, (5, 4, 4))
+    x2 = random.uniform(key2, (5, 4, 4))
 
     def vmap(fn):
       return nn.vmap(
@@ -259,19 +278,19 @@ class TransformTest(parameterized.TestCase):
 
     normal_model = decorated_MLP()(features=[3, 4, 5])
     vmap_model = decorated_MLP(vmap)(features=[3, 4, 5])
-    init_variables = normal_model.init(key2, x)
+    init_variables = normal_model.init(key3, x)
     # simulate vmap in python for comparison:
     y1 = jnp.vstack([
         normal_model.apply(init_variables, x2[i])[None, ...]
         for i in np.arange(x2.shape[0])
     ])
     y2 = vmap_model.apply(init_variables, x2)
-    np.testing.assert_allclose(y1, y2, atol=1e-7)
+    np.testing.assert_allclose(y1, y2, atol=1e-6)
 
   def test_vmap_batchnorm(self):
-    key1, key2 = random.split(random.key(3), 2)
+    key1, key2, key3 = random.split(random.key(3), 3)
     x = random.uniform(key1, (4, 4))
-    x2 = random.uniform(key1, (5, 4, 4))
+    x2 = random.uniform(key2, (5, 4, 4))
 
     def vmap(cls):
       return nn.vmap(
@@ -293,7 +312,7 @@ class TransformTest(parameterized.TestCase):
 
     normal_model = MlpBn()
     vmap_model = vmap(MlpBn)(axis_name='batch')
-    init_variables = normal_model.init(key2, x)
+    init_variables = normal_model.init(key3, x)
     y1 = normal_model.apply(
         init_variables, x2.reshape((-1, 4)), mutable=['batch_stats']
     )[0]
@@ -1575,6 +1594,7 @@ class TransformTest(parameterized.TestCase):
       def helper(self, x, ms):
         return ms[0](x) + ms[1](x)
 
+      @nn.fold_rngs
       def __call__(self, x):
         return self.helper(x, self.inners)
 
@@ -1583,7 +1603,6 @@ class TransformTest(parameterized.TestCase):
       def setup(self):
         self.inners = [nn.Dense(2), nn.Dense(2)]
 
-      @nn.jit
       def helper(self, x, ms):
         return ms[0](x) + ms[1](x)
 
@@ -1739,6 +1758,7 @@ class TransformTest(parameterized.TestCase):
       def setup_helper(self):
         self.b = nn.Dense(2)
 
+      @nn.fold_rngs
       def __call__(self, x):
         return self.b(self.a(x))
 
@@ -1792,6 +1812,87 @@ class TransformTest(parameterized.TestCase):
     self.assertEqual(s, 'hi')
     np.testing.assert_array_equal(y, jnp.array(1.0))
 
+  def test_jit_and_sow(self):
+    class Inner(nn.Module):
+
+      @nn.compact
+      def __call__(self, x):
+        self.sow('intermediates', 'loss', jnp.sum(x))
+        return x + 1
+
+    class Outer(nn.Module):
+
+      def setup(self):
+        self.inner = Inner()
+
+      @nn.jit
+      def __call__(self, x):
+        return self.inner(x)
+
+    m = Outer()
+    x = jnp.ones((2, 2))
+    vs = m.init(random.key(0), x)
+    y, updates = m.apply(vs, x, mutable=['intermediates'])
+    np.testing.assert_array_equal(
+        updates['intermediates']['inner']['loss'], 4.0
+    )
+    np.testing.assert_array_equal(y, 2)
+
+  def test_fold_rngs(self):
+    class Foo(nn.Module):
+
+      def __call__(self, use_jit: bool):
+        def f(foo: Foo):
+          return foo.make_rng('params')
+
+        if use_jit:
+          key = nn.jit(f)(self)
+        else:
+          key = nn.fold_rngs(f)(self)
+
+        return key
+
+    foo = Foo()
+    key_jit = foo.apply({}, True, rngs={'params': random.key(0)})
+    key_fold_rngs = foo.apply({}, False, rngs={'params': random.key(0)})
+
+    self.assert_keys_equal(key_jit, key_fold_rngs)
+
+  def test_same_key(self):
+
+    class Block(nn.Module):
+
+      @nn.jit
+      @nn.compact
+      def __call__(self, carry, inputs):
+        # dump_rng_info(self)
+        key = self.make_rng('params')
+        # y = jax.random.uniform(self.make_rng('params'), (2,))
+        return carry, key
+
+    class Transformer(nn.Module):
+
+      @nn.compact
+      def __call__(self):
+        num_blocks = 10
+        carry, key = nn.scan(
+            Block,
+            variable_axes={'params': 0},
+            split_rngs={'params': True},
+            # length=num_blocks,
+        )()(None, jnp.arange(num_blocks))
+        return key
+
+    model = Transformer()
+    keys1, _ = model.init_with_output(jax.random.key(1))
+    keys2, _ = model.init_with_output(jax.random.key(1))
+    keys3, _ = model.init_with_output(jax.random.key(1))
+    keys4, _ = model.init_with_output(jax.random.key(1))
+
+    self.assert_keys_equal(keys1, keys2)
+    self.assert_keys_equal(keys2, keys3)
+    self.assert_keys_equal(keys2, keys3)
+
   def test_jit_repr_hash(self):
     n = 0
 
@@ -1801,14 +1902,14 @@ class TransformTest(parameterized.TestCase):
       n += 1
       return None
 
-    f(_HashableProxy(nn.Dense(10)))
+    f(_HashableProxy.from_module(nn.Dense(10)))
     self.assertEqual(n, 1)
-    f(_HashableProxy(nn.Dense(10)))
+    f(_HashableProxy.from_module(nn.Dense(10)))
     self.assertEqual(n, 1)
 
-    f(_HashableProxy(nn.Dense(20)))
+    f(_HashableProxy.from_module(nn.Dense(20)))
     self.assertEqual(n, 2)
-    f(_HashableProxy(nn.Dense(20)))
+    f(_HashableProxy.from_module(nn.Dense(20)))
     self.assertEqual(n, 2)
 
   def test_jit_reuse(self):
@@ -1832,6 +1933,29 @@ class TransformTest(parameterized.TestCase):
     y = m.apply({}, x)
     self.assertEqual(n, 1)
 
+  def test_jit_recursive(self):
+    n = 0
+
+    class Foo(nn.Module):
+
+      @partial(nn.jit, static_argnames='recurse_once')
+      def __call__(self, x, *, recurse_once: bool = True):
+        nonlocal n
+        n += 1
+        if recurse_once:
+          x = self(x, recurse_once=False)
+        return x + 1
+
+    x = jnp.array(1.0)
+    m = Foo()
+
+    self.assertEqual(n, 0)
+
+    y = m.apply({}, x)
+    self.assertEqual(n, 2)
+    y = m.apply({}, x)
+    self.assertEqual(n, 2)
+
   @parameterized.named_parameters(('class', True), ('method', False))
   def test_jit_reuse_hash(self, jit_class: bool):
     n = 0
@@ -1900,6 +2024,54 @@ class TransformTest(parameterized.TestCase):
 
     y = Parent().apply({}, x)
 
+  @parameterized.named_parameters(('class', True), ('method', False))
+  def test_jit_stateful_submodules(self, jit_class: bool):
+    n = 0
+
+    class Foo(nn.Module):
+      key: int
+
+      @nn.compact
+      def __call__(self, x):
+        nonlocal n
+        n += 1
+        count = self.variable('counts', 'count', lambda: 0)
+        if not self.is_initializing():
+          count.value += 1
+        return x
+
+    if jit_class:
+      Foo = nn.jit(Foo)
+    else:  # jit method
+      Foo.__call__ = nn.jit(Foo.__call__)
+
+    class Parent(nn.Module):
+      @nn.compact
+      def __call__(self, x):
+        for _ in range(3):
+          m = Foo(0)
+          x = m(x)
+        return x
+
+    m = Parent()
+    x = jnp.array(1.0)
+    counts = m.init({}, x)['counts']
+    self.assertEqual(n, 1)
+
+    y, updates = m.apply({'counts': counts}, x, mutable=['counts'])
+    counts = updates['counts']
+    self.assertEqual(n, 2)
+
+    for count in jax.tree.leaves(counts):
+      self.assertEqual(count, 1)
+
+    y, updates = m.apply({'counts': counts}, x, mutable=['counts'])
+    counts = updates['counts']
+    self.assertEqual(n, 2)
+
+    for count in jax.tree.leaves(counts):
+      self.assertEqual(count, 2)
+
   @parameterized.named_parameters(('class', True), ('method', False))
   def test_jit_reuse_nested_submodules(self, jit_class: bool):
     test = self
@@ -1948,7 +2120,10 @@ class TransformTest(parameterized.TestCase):
 
       def __hash__(self):
         # test object is not being passed as static
-        raise Exception('immutable')
+        raise Exception('this should not be called')
+
+      def __eq__(self, __value, /):
+        raise Exception('this should not be called')
 
     def to_dict(node: Node):
       return {'a': node.a}
@@ -2032,14 +2207,16 @@ class TransformTest(parameterized.TestCase):
         rngs={'params': random.key(1), 'loop': random.key(2)},
     )
     self.assertEqual(vars['state']['acc'], x)
-    np.testing.assert_array_equal(
-        vars['state']['rng_params'][0], vars['state']['rng_params'][1]
-    )
-    np.testing.assert_array_compare(
-        operator.__ne__,
-        vars['state']['rng_loop'][0],
-        vars['state']['rng_loop'][1],
+    self.assertTrue(
+      jnp.equal(vars['state']['rng_params'][0], vars['state']['rng_params'][1])
     )
+    with jax.debug_key_reuse(False):
+      self.assertFalse(
+        jnp.equal(
+          vars['state']['rng_loop'][0],
+          vars['state']['rng_loop'][1],
+        )
+      )
 
   def test_cond(self):
     class Foo(nn.Module):
@@ -2245,7 +2422,7 @@ class TransformTest(parameterized.TestCase):
     x = jnp.ones((4, 4), dtype=jnp.float32)
     vs = Test().init(k, x)
     y = Test().apply(vs, x)
-    outer_expect = jax.tree_map(
+    outer_expect = jax.tree_util.tree_map(
         jnp.shape,
         freeze({
             'params': {
@@ -2259,7 +2436,7 @@ class TransformTest(parameterized.TestCase):
             }
         }),
     )
-    inner_expect = jax.tree_map(
+    inner_expect = jax.tree_util.tree_map(
         jnp.shape,
         freeze({
             'params': {
@@ -2271,8 +2448,8 @@ class TransformTest(parameterized.TestCase):
             }
         }),
     )
-    self.assertEqual(jax.tree_map(jnp.shape, vs), outer_expect)
-    self.assertEqual(jax.tree_map(jnp.shape, vars_copy), inner_expect)
+    self.assertEqual(jax.tree_util.tree_map(jnp.shape, vs), outer_expect)
+    self.assertEqual(jax.tree_util.tree_map(jnp.shape, vars_copy), inner_expect)
 
   def test_outer_setup_called_with_sharing_across_transforms(self):
     class A(nn.Module):
@@ -2305,10 +2482,10 @@ class TransformTest(parameterized.TestCase):
     x = random.randint(k, (2, 2), minval=0, maxval=10)
     vs = C().init(k, x)
     y = C().apply(vs, x)
-    outer_expect = jax.tree_map(
+    outer_expect = jax.tree_util.tree_map(
         jnp.shape, freeze({'params': {'a': {'foo': jnp.zeros((2, 2))}}})
     )
-    self.assertEqual(jax.tree_map(jnp.shape, vs), outer_expect)
+    self.assertEqual(jax.tree_util.tree_map(jnp.shape, vs), outer_expect)
 
   def test_grad_simple(self):
     class LearnScale(nn.Module):
@@ -2477,6 +2654,114 @@ class TransformTest(parameterized.TestCase):
     self.assertTrue(tree_allclose(jax.grad(comparison_fn, 0)(x, y), x_grad))
     self.assertTrue(tree_allclose(jax.grad(comparison_fn, 1)(x, y), y_grad))
 
+  def test_vmap_add_remove_axis_transforms(self):
+    class BoxedData(struct.PyTreeNode, AxisMetadata):
+      value: Any
+      def unbox(self):
+        return self.value
+      def replace_boxed(self, val):
+        return self.replace(value=val)
+      def add_axis(self, index: int, params: dict[Any, Any]):
+        value = jnp.mean(self.value, axis=index)
+        return self.replace(value=value)
+      def remove_axis(self, index: int, params: dict[Any, Any]):
+        value_shape = list(self.value.shape)
+        value_shape.insert(index, params['axis_size'])
+        value = jnp.broadcast_to(self.value, value_shape)
+        return self.replace(value=value)
+
+    class Top(nn.Module):
+      @nn.compact
+      def __call__(self, x):
+        VFoo = nn.vmap(
+            Foo, in_axes=0, out_axes=0, variable_axes={'params':0, 'aux': 0},
+            metadata_params={'axis_size': x.shape[0]},
+        )
+        vfoo = VFoo(name="vfoo")
+        y = vfoo(x)
+        y = vfoo(x)
+        assert vfoo.variables['aux']['v'].value.shape == ()
+        return y
+
+    class Foo(nn.Module):
+      @nn.compact
+      def __call__(self, x):
+        if self.has_variable('aux', 'v'):
+          assert self.variables['aux']['v'].value.shape == ()
+        boxed_v = self.variable('aux', 'v', lambda: BoxedData(jnp.ones(())))
+        assert self.variables['aux']['v'].value.shape == ()
+        return x
+
+    vs = Top().init(random.key(0), jnp.ones((2,5)))
+    y = Top().apply(vs, jnp.ones((2, 5)))
+    assert vs['aux']['vfoo']['v'].value.shape == ()
+
+  def test_vjp_tracer_leak(self):
+    class LearnScale(nn.Module):
+      @nn.compact
+      def __call__(self, x):
+        p = self.param('scale', nn.initializers.zeros, ())
+        return p * x
+    class Foo(nn.Module):
+      @nn.compact
+      def __call__(self, x):
+        y, bwd = nn.vjp(lambda mdl, x: mdl(x), LearnScale(), x)
+        params_grad, x_grad = bwd(jnp.ones(y.shape))
+        return y, params_grad, x_grad
+    key = jax.random.PRNGKey(0)
+    x = jnp.ones((2, 3))
+    foo = Foo()
+    with jax.check_tracer_leaks():
+      params = foo.init(key, x)
+      foo.apply(params, x)
+
+  @parameterized.named_parameters(
+      ('retracing scan', True), ('simple scan', False)
+  )
+  def test_jit_scan_retracing(self, retracing_scan: bool):
+    num_blocks = 4
+    num_patterns = 4
+    features = 4
+    trace_counts = [0, 0]
+
+    class Block(nn.Module):
+      def setup(self):
+        self.dense = nn.Dense(features, use_bias=False)
+      @nn.jit
+      def __call__(self, x):
+        nonlocal trace_counts
+        trace_counts[1] += 1
+        return self.dense(x)
+
+    class BlockSequence(nn.Module):
+      def setup(self):
+        self.blocks = [Block() for _ in range(num_blocks)]
+      @nn.jit
+      def __call__(self, carry, inputs):
+        nonlocal trace_counts
+        trace_counts[0] += 1
+        for block in self.blocks:
+          carry = block(carry)
+        return carry, inputs
+
+    class Transformer(nn.Module):
+      retracing_scan: bool = True
+      def setup(self):
+        self.scan = nn.scan(
+            BlockSequence,
+            variable_axes={'params': 0},
+            split_rngs={'params': False},
+            length=num_patterns,
+            check_constancy_invariants=retracing_scan,
+        )()
+      def __call__(self, inputs):
+        return self.scan(jnp.zeros_like(inputs), inputs)
+
+    model = Transformer(retracing_scan=retracing_scan)
+    _ = model.init(random.key(0), jnp.ones((num_patterns, features,)))
+    self.assertEqual(trace_counts[0], 2 if retracing_scan else 1)
+    self.assertEqual(trace_counts[1], 2 if retracing_scan else 1)
+
 
 if __name__ == '__main__':
   absltest.main()
